import sys

from six.moves import cPickle

def __copy(d):
    # Faster than deepcopy - ideally remove the need for deepcopy altogether
    return cPickle.loads(cPickle.dumps(d, protocol=2))

class Node(object):
    def __init__(self, tp, children=None, value=None):
        self.type = tp
        if children:
            self.children = children
        else:
            self.children = []
        self.value = value

    def __str__(self):
        return "Node(%r)" % self.type

    def __repr__(self):
        return "Node(%r)" % self.type

def ast_pprint(root, cur_ind=0, ind_val=4, string=None):
    """Pretty printer for the yacc-based parser."""
    _buf = []

    def _ast_pprint(_root, _cur_ind):
        if not hasattr(_root, "children"):
            _buf.append(str(_root))
        else:
            if _root.children:
                _buf.append("%sNode(type='%s'):" % (' ' * _cur_ind * ind_val,
                                                    _root.type))
                for c in _root.children:
                    _ast_pprint(c, _cur_ind + 1)
            else:
                msg = "%sNode(type='%s'" % (' ' * _cur_ind * ind_val,
                                            _root.type)
                if _root.value is not None:
                    msg += ", value=%r)" % _root.value
                else:
                    msg += ")"
                _buf.append(msg)

    _ast_pprint(root, cur_ind)
    if string is None:
        print("\n".join(_buf))
    else:
        string.write("\n".join(_buf))

def ast_walk(root, dispatcher, debug=False):
    """Walk the given tree and for each node apply the corresponding function
    as defined by the dispatcher.
    
    If one node type does not have any function defined for it, it simply
    returns the node unchanged.
    
    Parameters
    ----------
    root : Node
        top of the tree to walk into.
    dispatcher : Dispatcher
        defines the action for each node type.
    """
    def _walker(par):
        children = []
        for c in par.children:
            children.append(_walker(c))

        par.children = [c for c in children if c is not None]
        try:
            func = dispatcher.action_dict[par.type]
            return func(par)
        except KeyError:
            if debug:
                print("no action for type %s" % par.type)
            return par

    # FIXME: we need to copy the dict because the dispatcher modify the dict in
    # place ATM, and we call this often. This is very expensive, and should be
    # removed as it has a huge cost in starting time (~30 % in hot case)
    return _walker(__copy(root))



import sys

import os.path as op

_BACKENDS = {
        "Waf": ("WafBackend", "waf_backend"),
        "Yaku": ("YakuBackend", "yaku_backend"),
        "Distutils": ("DistutilsBackend", "distutils_backend")
}

def load_backend(backend_name, backend_dirs=None):
    if backend_dirs is None:
        backend_dirs = [op.dirname(__file__)]
    else:
        backend_dirs = [op.dirname(__file__)] + backend_dirs

    if backend_name in _BACKENDS:
        class_name, module_name = _BACKENDS[backend_name]
        sys.path = backend_dirs + sys.path
        try:
            __import__(module_name)
            mod = sys.modules[module_name]
            backend = getattr(mod, class_name, None)
            if backend:
                return backend
            else:
                return ValueError("Backend class %r not found in module %r" % (class_name, mod))
        finally:
            for d in backend_dirs:
                sys.path.remove(d)
    else:
        raise ValueError("Unrecognized backend: %r" % (backend_name,))



import imp
import os
import re
import sys
import traceback

from six.moves \
    import \
        StringIO

from bento.commands.core \
    import \
        Command
from bento.compat \
    import \
        inspect as compat_inspect
from bento.utils.utils \
    import \
        extract_exception
from bento.errors \
    import \
        InvalidHook

SAFE_MODULE_NAME = re.compile("[^a-zA-Z_]")

class HookRegistry(object):
    def __init__(self):
        self._pre_hooks = {}
        self._post_hooks = {}

    def add_pre_hook(self, hook, cmd_name):
        if cmd_name in self._pre_hooks:
            self._pre_hooks[cmd_name].append(hook)
        else:
            self._pre_hooks[cmd_name] = [hook]

    def add_post_hook(self, hook, cmd_name):
        if cmd_name in self._post_hooks:
            self._post_hooks[cmd_name].append(hook)
        else:
            self._post_hooks[cmd_name] = [hook]

    def retrieve_pre_hooks(self, cmd_name):
        return self._pre_hooks.get(cmd_name, [])

    def retrieve_post_hooks(self, cmd_name):
        return self._post_hooks.get(cmd_name, [])

def find_pre_hooks(modules, cmd_name):
    """Retrieve all pre hooks instances defined in given modules list.

    This should be used to find prehooks defined through the hook.pre_*. This
    works by looking for all WrappedCommand instances in the modules.

    Parameters
    ----------
    modules: seq
        list of modules to look into
    cmd_name: str
        command name
    """
    pre_hooks = []
    for module in modules:
        pre_hooks.extend([f for f in vars(module).values() if isinstance(f,
            PreHookWrapper) and f.cmd_name == cmd_name])
    return pre_hooks

def find_post_hooks(modules, cmd_name):
    """Retrieve all post hooks instances defined in given modules list.

    This should be used to find prehooks defined through the hook.pre_*. This
    works by looking for all WrappedCommand instances in the modules.

    Parameters
    ----------
    modules: seq
        list of modules to look into
    cmd_name: str
        command name
    """
    post_hooks = []
    for module in modules:
        post_hooks.extend([f for f in vars(module).values() if isinstance(f,
            PostHookWrapper) and f.cmd_name == cmd_name])
    return post_hooks

def find_startup_hooks(modules):
    """Retrieve all startup hook instances defined in given modules list.

    This should be used to find hooks defined through the hook.startup.

    Parameters
    ----------
    modules: seq
        list of modules to look into
    """
    hooks = []
    for module in modules:
        hooks.extend([f for f in vars(module).values() if isinstance(f, StartupHook)])
    return hooks

def find_shutdown_hooks(modules):
    """Retrieve all shutdown hook instances defined in given modules list.

    This should be used to find hooks defined through the hook.shutdown.

    Parameters
    ----------
    modules: seq
        list of modules to look into
    """
    hooks = []
    for module in modules:
        hooks.extend([f for f in vars(module).values() if isinstance(f, ShutdownHook)])
    return hooks

def find_options_hooks(modules):
    """Retrieve all options hook instances defined in given modules list.

    This should be used to find hooks defined through the hook.options.

    Parameters
    ----------
    modules: seq
        list of modules to look into
    """
    hooks = []
    for module in modules:
        hooks.extend([f for f in vars(module).values() if isinstance(f, OptionsHook)])
    return hooks

class _HookWrapperBase(object):
    def __init__(self, func, cmd_name, local_dir):
        self._func = func
        self.local_dir = local_dir
        self.cmd_name = cmd_name
        self.name = func.__name__

    def __call__(self, ctx):
        return self._func(ctx)

    def __getattr__(self, k):
        return getattr(self._func, k)

class PreHookWrapper(_HookWrapperBase):
    pass

class PostHookWrapper(_HookWrapperBase):
    pass

class _GlobalHookBase(object):
    def __init__(self, func):
        self._func = func

    def __call__(self, ctx):
        return self._func(ctx)

    def __getattr__(self, k):
        return getattr(self._func, k)

class StartupHook(_GlobalHookBase):
    pass

class ShutdownHook(_GlobalHookBase):
    pass

class OptionsHook(_GlobalHookBase):
    pass

def _make_hook_decorator(command_name, kind):
    def decorator(f):
        local_dir = os.path.dirname(compat_inspect.stack()[1][1])
        if kind == "post":
            hook = PostHookWrapper(f, command_name, local_dir)
        elif kind == "pre":
            hook = PreHookWrapper(f, command_name, local_dir)
        else:
            raise ValueError("invalid hook kind %s" % kind)
        return hook
    return decorator

post_configure = _make_hook_decorator("configure", "post")
pre_configure = _make_hook_decorator("configure", "pre")
post_build = _make_hook_decorator("build", "post")
pre_build = _make_hook_decorator("build", "pre")
post_sdist = _make_hook_decorator("sdist", "post")
pre_sdist = _make_hook_decorator("sdist", "pre")

def options(f):
    return OptionsHook(f)

def startup(f):
    return StartupHook(f)

def shutdown(f):
    return ShutdownHook(f)

def create_hook_module(target):
    safe_name = SAFE_MODULE_NAME.sub("_", target, len(target))
    module_name = "bento_hook_%s" % safe_name
    main_file = os.path.abspath(target)
    module = imp.new_module(module_name)
    module.__file__ = main_file
    code = open(main_file).read()

    sys.path.insert(0, os.path.dirname(main_file))
    try:
        exec(compile(code, main_file, 'exec'), module.__dict__)
        sys.modules[module_name] = module
    except Exception:
        sys.path.pop(0)
        e = extract_exception()
        tb = sys.exc_info()[2]
        s = StringIO()
        traceback.print_tb(tb, file=s)
        msg = """\
Could not import hook file %r: caught exception %r
Original traceback (most recent call last)
%s\
""" % (main_file, e, s.getvalue())
        raise InvalidHook(msg)

    module.root_path = main_file
    return module

class WrappedCommand(Command):
    def __init__(self, func):
        super(WrappedCommand, self).__init__()
        self._func = func
        self.name = func.__name__

    def __call__(self, ctx):
        return self.run(ctx)

    def run(self, ctx):
        return self._func(ctx)

    def __getattr__(self, k):
        return getattr(self._func, k)

def command(f):
    """Decorator to create a new command from a simple function

    The function should take one CommandContext instance

    Example
    -------
 
    A simple command may be defined as follows::

        @command
        def hello(context):
            print "hello"
    """
    return WrappedCommand(f)

def find_command_hooks(modules):
    """Retrieve all command instances defined in given modules list.

    This should be used to find commands defined through the hook.command. This
    works by looking for all WrappedCommand instances in the modules.

    Parameters
    ----------
    modules: seq
        list of modules to look into
    """
    commands = []
    for module in modules:
        commands.extend([f for f in vars(module).values() if isinstance(f,
            WrappedCommand)])
    return commands



import sys
if sys.version_info >= (3,):
    import builtins
    if not hasattr(builtins, "WindowsError"):
        class WindowsError(Exception):
            pass
    else:
        WindowsError = builtins.WindowsError
else:
    import __builtin__
    if not hasattr(__builtin__, "WindowsError"):
        class WindowsError(Exception):
            pass
    else:
        WindowsError = __builtin__.WindowsError

class YakuError(Exception):
    pass

class TaskRunFailure(YakuError):
    def __init__(self, cmd, explain=None):
        self.cmd = cmd
        self.explain = explain

    def __str__(self):
        ret = "Command failed. Error was: \n\t%s\ncommand line was\n\t%r" % (self.explain.rstrip(),
                                                                          " ".join(self.cmd))
        if sys.version_info < (3,):
            return ret.encode("utf-8")
        else:
            return ret

class ConfigurationFailure(YakuError):
    pass

class UnknownTask(YakuError):
    pass

class ToolNotFound(YakuError):
    pass



from bento.commands.configure \
    import \
        _compute_scheme, set_scheme_options
from bento.commands.registries \
    import \
        CommandRegistry, ContextRegistry, OptionsRegistry
from bento.commands.dependency \
    import \
        CommandScheduler
from bento.commands.hooks \
    import \
        HookRegistry
from bento.utils.utils \
    import \
        read_or_create_dict

from six.moves \
    import \
        cPickle

class GlobalContext(object):
    def __init__(self, command_data_db, commands_registry=None, contexts_registry=None,
            options_registry=None, commands_scheduler=None):
        self._commands_registry = commands_registry or CommandRegistry()
        self._contexts_registry = contexts_registry or ContextRegistry()
        self._options_registry = options_registry or OptionsRegistry()
        self._scheduler = commands_scheduler or CommandScheduler()
        self._hooks_registry = HookRegistry()

        self.backend = None
        self._package_options = None

        self._command_data_db = command_data_db
        if command_data_db is None:
            self._command_data_store = {}
        else:
            self._command_data_store = read_or_create_dict(command_data_db.abspath())

    def store(self):
        if self._command_data_db:
            self._command_data_db.safe_write(cPickle.dumps(self._command_data_store), "wb")

    #------------
    # Command API
    #------------
    def register_command(self, cmd_name, cmd, public=True):
        """Register a command name to a command instance.

        Parameters
        ----------
        cmd_name: str
            name of the command
        cmd: object
            instance from a subclass of Command
        """
        self._commands_registry.register(cmd_name, cmd, public)

    def retrieve_command(self, cmd_name):
        """Return the command instance registered for the given command name."""
        return self._commands_registry.retrieve(cmd_name)

    def is_command_registered(self, cmd_name):
        """Return True if the command is registered."""
        return self._commands_registry.is_registered(cmd_name)

    def command_names(self, public_only=True):
        if public_only:
            return self._commands_registry.public_command_names()
        else:
            return self._commands_registry.command_names()

    #--------------------
    # Command Context API
    #--------------------
    def register_command_context(self, cmd_name, klass):
        self._contexts_registry.register(cmd_name, klass)

    def retrieve_command_context(self, cmd_name):
        return self._contexts_registry.retrieve(cmd_name)

    def is_command_context_registered(self, cmd_name):
        """Return True if the command context is registered."""
        return self._contexts_registry.is_registered(cmd_name)

    #--------------------
    # Command Options API
    #--------------------
    def register_options_context_without_command(self, name, context):
        """Register options_context for special 'commands' that has no command
        context attached to them

        This is typically used for global help, and other help-only commands
        such as 'globals'."""
        return self._options_registry.register(name, context)

    def register_options_context(self, cmd_name, context):
        cmd = self.retrieve_command(cmd_name)
        if self._package_options is not None and hasattr(cmd, "register_options"):
            cmd.register_options(context, self._package_options)

        return self._options_registry.register(cmd_name, context)

    def retrieve_options_context(self, cmd_name):
        return self._options_registry.retrieve(cmd_name)

    def is_options_context_registered(self, cmd_name):
        return self._options_registry.is_registered(cmd_name)

    def add_option_group(self, cmd_name, name, title):
        """Add a new option group for the given command.
        
        Parameters
        ----------
        cmd_name: str
            name of the command
        name: str
            name of the group option
        title: str
            title of the group
        """
        ctx = self._options_registry.retrieve(cmd_name)
        ctx.add_group(name, title)

    def add_option(self, cmd_name, option, group=None):
        """Add a new option for the given command.

        Parameters
        ----------
        cmd_name: str
            name of the command
        option: str
            name of the option
        group: str, None
            group to associated with
        """
        ctx = self._options_registry.retrieve(cmd_name)
        ctx.add_option(option, group)

    def register_package_options(self, package_options):
        self._package_options = package_options

    # FIXME: rename
    def retrieve_scheme(self):
        """Return the path scheme, including any custom path defined in the
        bento.info script (Path sections)."""
        return _compute_scheme(self._package_options)

    def retrieve_configured_scheme(self, command_argv=None):
        """Return the configured path scheme with the given command argv.

        Note
        ----
        This can only be safely called once regiser_options_context has been
        called for the configure command"""
        assert self._package_options is not None
        assert self.is_options_context_registered("configure")

        if command_argv is None:
            command_argv = []
        scheme = _compute_scheme(self._package_options)
        options_context = self.retrieve_options_context("configure")
        o, a = options_context.parser.parse_args(command_argv)
        set_scheme_options(scheme, o, self._package_options)
        return scheme

    #-----------------------
    # Command dependency API
    #-----------------------
    def set_before(self, cmd_name, cmd_name_before):
        """Specify that command cmd_name_before should run before cmd_name."""
        self._scheduler.set_before(cmd_name, cmd_name_before)

    def set_after(self, cmd_name, cmd_name_after):
        """Specify that command cmd_name_before should run after cmd_name."""
        self._scheduler.set_after(cmd_name, cmd_name_after)

    def retrieve_dependencies(self, cmd_name):
        """Return the ordered list of command names to run before the given
        command name."""
        return self._scheduler.order(cmd_name)

    #---------
    # Hook API
    #---------
    def add_pre_hook(self, hook, cmd_name):
        self._hooks_registry.add_pre_hook(hook, cmd_name)

    def add_post_hook(self, hook, cmd_name):
        self._hooks_registry.add_post_hook(hook, cmd_name)

    def retrieve_pre_hooks(self, cmd_name):
        return self._hooks_registry.retrieve_pre_hooks(cmd_name)

    def retrieve_post_hooks(self, cmd_name):
        return self._hooks_registry.retrieve_post_hooks(cmd_name)

    #------------
    # Backend API
    #------------
    def register_backend(self, backend):
        self.backend = backend

    def retrieve_command_argv(self, command_name):
        # FIXME: returning empty list if nothing is available hides bugs.
        return self._command_data_store.get(command_name, [])

    def save_command_argv(self, command_name, command_argv):
        self._command_data_store[command_name] = command_argv



from bento.utils.os2 \
    import \
        rename

def safe_write(target, writer, mode="wb"):
    """a 'safe' way to write to files.

    Instead of writing directly into a file, this function writes to a
    temporary file, and then rename the file to the target. On sane
    platforms, rename is atomic, so this avoids leaving stale
    files in inconsistent states.

    Parameters
    ----------
    target: str
        destination to write to
    writer: callable
        function which takes one argument, a file descriptor, and
        writes content to it
    mode: str
        opening mode
    """
    f = open(target + ".tmp", mode)
    try:
        writer(f)
    finally:
        f.close()
        rename(target + ".tmp", target)



from bento.core.package \
    import \
        PackageDescription
from bento.core.meta \
    import \
        PackageMetadata
from bento.core.options \
    import \
        PackageOptions

__all__ = ["PackageDescription", "PackageMetadata", "PackageOptions"]



import sys
import copy

from bento.parser.nodes \
    import \
        Node

def split_newline(s):
    try:
        ind = [i.type for i in s].index("newline")
        return [s[:ind+1], s[ind+1:]]
    except ValueError:
        return [s]

def split_newlines(s):
    t = []
    def _split_newlines(s):
        sp = split_newline(s)
        t.append(sp[0])
        if len(sp) > 1 and sp[1]:
            _split_newlines(sp[1])

    _split_newlines(s)
    return t

# XXX: fix the str vs bool issue with flag variables
_LIT_BOOL = {"true": True, "false": False, True: True, False: False}

class Dispatcher(object):
    def __init__(self, user_values=None):
        self._d = {
                "path_options": {},
                "flag_options": {},
                "libraries": {},
                "executables": {},
                "data_files": {},
                "extra_source_files": [],
                "hook_files": [],
        }
        self.action_dict = {
            "empty": self.empty,
            "stmt_list": self.stmt_list,
            "description": self.description,
            "description_from_file": self.description_from_file,
            "summary": self.summary,
            "author": self.author,
            "maintainer": self.maintainer,
            "hook_files": self.hook_files,
            "config_py": self.config_py,
            "meta_template_file": self.meta_template_file,
            "subento": self.subento,
            "use_backends": self.use_backends,
            # Library
            "library": self.library,
            "library_name": self.library_name,
            "library_stmts": self.library_stmts,
            # Path
            "path": self.path,
            "path_default": self.path_default,
            "path_stmts": self.path_stmts,
            "path_description": self.path_description,
            # Flag
            "flag": self.flag,
            "flag_default": self.flag_default,
            "flag_stmts": self.flag_stmts,
            "flag_description": self.flag_description,
            # Extension
            "extension": self.extension,
            "extension_declaration": self.extension_declaration,
            "extension_field_stmts": self.extension_field_stmts,
            # Pure C library
            "compiled_library": self.compiled_library,
            "compiled_library_declaration": self.compiled_library_declaration,
            "compiled_library_field_stmts": self.compiled_library_field_stmts,
            # Conditional
            "conditional": self.conditional,
            "osvar": self.osvar,
            "flagvar": self.flagvar,
            "not_flagvar": self.not_flagvar,
            "bool": self.bool_var,
            # Extra source files
            "extra_source_files": self.extra_source_files,
            # Data files handling
            "data_files": self.data_files,
            "data_files_stmts": self.data_files_stmts,
            # Executable
            "executable": self.executable,
            "exec_stmts": self.exec_stmts,
            "exec_name": self.exec_name,
            "function": self.function,
            "module": self.module,
            "sub_directory": self.sub_directory,
        }
        if user_values is not None:
            self._vars = copy.deepcopy(user_values)
        else:
            self._vars = {}

    def empty(self, node):
        return {}

    def stmt_list(self, node):
        for c in node.children:
            if c.type in ["name", "description", "version", "summary", "url",
                          "download_url", "author", "author_email",
                          "maintainer", "maintainer_email", "license",
                          "platforms", "classifiers", "hook_files",
                          "config_py", "description_from_file",
                          "meta_template_files", "keywords", "use_backends"]:
                self._d[c.type] = c.value
            elif c.type == "path":
                self._d["path_options"].update({c.value["name"]: c.value})
            elif c.type == "flag":
                self._d["flag_options"].update({c.value["name"]: c.value})
            elif c.type == "library":
                self._d["libraries"].update({c.value["name"]: c.value})
            elif c.type == "executable":
                self._d["executables"].update({c.value["name"]: c.value})
            elif c.type == "data_files":
                self._d["data_files"].update({c.value["name"]: c.value})
            else:
                raise ValueError("Unhandled top statement (%s)" % c)
        return self._d

    def summary(self, node):
        ret = Node(node.type)
        ret.value = "".join([i.value for i in node.value])
        return ret

    def author(self, node):
        node.value = "".join([i.value for i in node.value])
        return node

    def maintainer(self, node):
        return self.author(node)

    def hook_files(self, node):
        self._d["hook_files"].extend(node.value)

    def config_py(self, node):
        return node

    def meta_template_file(self, node):
        return node

    def description_from_file(self, node):
        return node

    def description(self, node):
        tokens = []
        for i in node.value:
            if i.type in ["literal", "multi_literal", "newline",
                          "indent", "dedent", "single_line"]:
                tokens.append(i)

        # FIXME: fix grammar to get ind_shift
        ind_shift = 4
        inds = [0]
        line_str = []
        for line in split_newlines(tokens):
            # FIXME: this is horrible
            if line[0].type == "dedent":
                if set([node.type for node in line]) == set(["dedent"]): # if the line only contains dedent
                    inds.pop(0)
                    remain = [Node("literal", value="\n")]
                else:
                    while line[0].type == "dedent":
                        inds.pop(0)
                        line = line[1:]
                    remain = line
            elif line[0].type == "indent":
                inds.insert(0, line[0].value - ind_shift)
                remain = line[1:]
            else:
                remain = line

            if len(remain) > 0:
                if remain[-1].type == "dedent":
                    remain = remain[:-1]

            cur_line = [" " * inds[0]]
            cur_line.extend([t.value for t in remain])
            line_str.append("".join(cur_line))

        return Node("description", value="".join(line_str))

    #--------------------------
    # Library section handlers
    #--------------------------
    def library(self, node):
        library = {"py_modules": [],
                   "install_requires": [],
                   "build_requires": [],
                   "packages": [],
                   "extensions": {},
                   "compiled_libraries": {},
                   "sub_directory": None,
                   }

        def update(library_dict, c):
            if type(c) == list:
                for i in c:
                    update(library_dict, i)
            elif c.type == "name":
                library_dict["name"] = c.value
            elif c.type == "modules":
                library_dict["py_modules"].extend(c.value)
            elif c.type == "packages":
                library_dict["packages"].extend(c.value)
            elif c.type in ("build_requires", "install_requires"):
                library_dict[c.type].extend(c.value)
            elif c.type == "extension":
                name = c.value["name"]
                library_dict["extensions"][name] = c.value
            elif c.type == "compiled_library":
                name = c.value["name"]
                library_dict["compiled_libraries"][name] = c.value
            elif c.type == "sub_directory":
                library_dict["sub_directory"] = c.value
            else:
                raise ValueError("Unhandled node type: %s" % c)

        if len(node.children) > 1:
            nodes = [node.children[0]] + node.children[1]
        else:
            nodes = [node.children[0]]
        for c in nodes:
            update(library, c)
        return Node("library", value=library)

    def library_name(self, node):
        return Node("name", value=node.value)

    def library_stmts(self, node):
        return node.children

    def extension(self, node):
        ret = {}
        seen = set()

        def _ensure_unique(field):
            if field in seen:
                raise ValueError("Field %r for extension %r is specified more than once !" % (field, ret["name"]))
            else:
                seen.add(field)

        def update(extension_dict, c):
            if type(c) == list:
                for i in c:
                    update(extension_dict, i)
            elif c.type == "name":
                ret["name"] = c.value
            elif c.type == "sources":
                _ensure_unique("sources")
                ret["sources"] = c.value
            elif c.type == "include_dirs":
                _ensure_unique("include_dirs")
                ret["include_dirs"] = c.value
            else:
                raise ValueError("Gne ?")
        for c in [node.children[0]] + node.children[1]:
            update(ret, c)
        return Node("extension", value=ret)

    def extension_field_stmts(self, node):
        return node.children

    def extension_declaration(self, node):
        return Node("name", value=node.value)

    def compiled_library(self, node):
        ret = {"sources": [], "include_dirs": []}
        seen = set()

        def _ensure_unique(field):
            if field in seen:
                raise ValueError("Field %r for compiled library %r is specified more than once !" % (field, ret["name"]))
            else:
                seen.add(field)

        def update(compiled_library_dict, c):
            if c.type == "name":
                ret["name"] = c.value
            elif c.type == "sources":
                _ensure_unique("sources")
                ret["sources"] = c.value
            elif c.type == "include_dirs":
                _ensure_unique("include_dirs")
                ret["include_dirs"] = c.value
            else:
                raise ValueError("Unknown node %s" % c)
        for c in [node.children[0]] + node.children[1]:
            update(ret, c)
        return Node("compiled_library", value=ret)

    def compiled_library_field_stmts(self, node):
        return node.children

    def compiled_library_declaration(self, node):
        return Node("name", value=node.value)

    def sub_directory(self, node):
        return Node("sub_directory", value=node.value)

    def use_backends(self, node):
        return Node("use_backends", value=node.value)

    #-----------------
    #   Path option
    #-----------------
    def path_stmts(self, node):
        return node.children

    def path(self, node):
        path = {}
        def update(c):
            if type(c) == list:
                for i in c:
                    update(i)
            elif c.type == "path_declaration":
                path["name"] = c.value
            elif c.type == "path_description":
                path["description"] = c.value
            elif c.type == "default":
                path["default"] = c.value
            else:
                raise SyntaxError("GNe ?")
        if len(node.children) > 1:
            nodes = [node.children[0]] + node.children[1]
        else:
            nodes = [node.children[0]]
        for node in nodes:
            update(node)

        if not "description" in path:
            raise ValueError("missing description in path section %r" %
                             (path["name"],))
        if not "default" in path:
            raise ValueError("missing default in path section %r" %
                             (path["name"],))
        return Node("path", value=path)

    def path_default(self, node):
        return Node("default", value=node.value)

    def path_stmts(self, node):
        return node.children

    def path_description(self, node):
        node.value = "".join([i.value for i in node.value])
        return node

    #-----------------
    #   Flag option
    #-----------------
    # XXX: refactor path/flag handling, as they are almost identical
    def flag(self, node):
        flag = {}
        for i in [node.children[0]] + node.children[1]:
            if i.type == "flag_declaration":
                flag["name"] = i.value
            elif i.type == "flag_description":
                flag["description"] = i.value
            elif i.type == "default":
                flag["default"] = i.value
            else:
                raise SyntaxError("GNe ?")

        if not flag["default"] in ["true", "false"]:
            raise SyntaxError("invalid default value %s for flag %s" \
                              % (flag["default"], flag["name"])) 

        if not flag["name"] in self._vars:
            self._vars[flag["name"]] = flag["default"]

        return Node("flag", value=flag)

    def flag_default(self, node):
        return Node("default", value=node.value)

    def flag_stmts(self, node):
        return node.children

    def flag_description(self, node):
        node.value = "".join([i.value for i in node.value])
        return node

    #-------------------
    #   Conditionals
    #-------------------
    def conditional(self, node):
        test = node.value
        if self.action_dict[test.type](test):
            return node.children[:1]
        else:
            return node.children[1:]

    def osvar(self, node):
        os_name = node.value.value
        return os_name == sys.platform

    def bool_var(self, node):
        return node.value

    def not_flagvar(self, node):
        name = node.value.value
        try:
            value = self._vars[name]
        except KeyError:
            raise ValueError("Unknown flag variable %s" % name)
        else:
            return not _LIT_BOOL[value]

    def flagvar(self, node):
        name = node.value.value
        try:
            value = self._vars[name]
        except KeyError:
            raise ValueError("Unknown flag variable %s" % name)
        else:
            return _LIT_BOOL[value]

    def extra_source_files(self, node):
        self._d["extra_source_files"].extend(node.value)

    def subento(self, node):
        if "subento" in self._d:
            self._d["subento"].extend(node.value)
        else:
            self._d["subento"] = node.value

    # Data handling
    def data_files(self, node):
        d = {}

        def update(data_d, c):
            if type(c) == list:
                for  i in c:
                    update(data_d, i)
            elif c.type == "data_files_declaration":
                d["name"] = c.value
            elif c.type == "source_dir":
                d["source_dir"] = c.value
            elif c.type == "target_dir":
                d["target_dir"] = c.value
            elif c.type == "files":
                d["files"] = c.value
            else:
                raise ValueError("Unhandled node type: %s" % c)

        for c in node.children:
            update(d, c)

        return Node("data_files", value=d)

    def data_files_stmts(self, node):
        return node.children

    # Executable handling
    def executable(self, node):
        d = {}

        def update(exec_d, c):
            if type(c) == list:
                for  i in c:
                    update(exec_d, i)
            elif c.type == "name":
                exec_d["name"] = c.value
            elif c.type == "module":
                exec_d["module"] = c.value
            elif c.type == "function":
                exec_d["function"] = c.value
            else:
                raise ValueError("Unhandled node type: %s" % c)

        for c in node.children:
            update(d, c)

        return Node("executable", value=d)

    def exec_stmts(self, node):
        return node.children

    def exec_name(self, node):
        return Node("name", value=node.value)

    def function(self, node):
        return Node("function", value=node.value)

    def module(self, node):
        return Node("module", value=node.value)



import shutil
import tempfile

from bento.compat.api.moves \
    import \
        unittest
from bento.core.node \
    import \
        create_first_node, find_root
import bento.core.pkg_objects as pkg_objects
import bento.core.package as package
import bento.core.subpackage as subpackage

def create_fake_tree(top_node, tree):
    for f in tree:
        n = top_node.make_node(f)
        n.parent.mkdir()
        n.write("")

class TestTranslation(unittest.TestCase):
    """Those tests check 'translation' from local definition (as
    defined in the subento file) to the top directory."""
    def test_extension(self):
        tree = [
            "foo/src/hellomodule.c",
            "foo/bento.info",
            "bento.info"]
        extension = pkg_objects.Extension("_hello",
                        sources=["src/hellomodule.c"],
                        include_dirs=["."])
        spkg = package.SubPackageDescription(
                        rdir="foo",
                        extensions={"_hello": extension})

        d = tempfile.mkdtemp()
        try:
            top = create_first_node(d)
            create_fake_tree(top, tree)
            extensions = subpackage.flatten_subpackage_extensions(
                    spkg, top)
            self.failUnless("foo._hello" in extensions)
            extension = extensions["foo._hello"]
            self.assertEqual(extension.sources, ["foo/src/hellomodule.c"])
            self.assertEqual(extension.include_dirs, ["foo"])
        finally:
            shutil.rmtree(d)

    def _test_compiled_library(self, tree, clib, spkg, sources, include_dirs):
        d = tempfile.mkdtemp()
        try:
            top = create_first_node(d)
            create_fake_tree(top, tree)
            clibs = subpackage.flatten_subpackage_compiled_libraries(
                    spkg, top)
            self.failUnless("bar.clib" in clibs)
            clib = clibs["bar.clib"]
            self.assertEqual(clib.sources, sources)
            self.assertEqual(clib.include_dirs, include_dirs)
        finally:
            shutil.rmtree(d)


    def test_compiled_library(self):
        tree = [
            "bar/src/clib.c",
            "bar/bento.info",
            "bento.info"]
        clib = pkg_objects.CompiledLibrary("clib",
                        sources=["src/clib.c"],
                        include_dirs=["."])
        spkg = package.SubPackageDescription(
                        rdir="bar",
                        compiled_libraries={"clib": clib})

        self._test_compiled_library(tree, clib, spkg, ["bar/src/clib.c"], ["bar"])

    def test_compiled_library_glob(self):
        tree = [
            "bar/src/clib.c",
            "bar/src/clib2.c",
            "bar/src/clib3.f",
            "bar/bento.info",
            "bento.info"]
        clib = pkg_objects.CompiledLibrary("clib",
                        sources=["src/*.c", "src/*.f"],
                        include_dirs=["."])
        spkg = package.SubPackageDescription(
                        rdir="bar",
                        compiled_libraries={"clib": clib})

        self._test_compiled_library(tree, clib, spkg, ["bar/src/clib.c", "bar/src/clib2.c", "bar/src/clib3.f"],
                                    ["bar"])



import os
import tempfile

from bento.compat.api.moves \
    import \
        unittest
from bento.core.package \
    import \
        PackageDescription
from bento.core.package import static_representation
from bento.core.meta import PackageMetadata
from bento.core.pkg_objects import DataFiles
from bento.core.node import Node

def create_file(file, makedirs=True):
    if makedirs:
        dirname = os.path.dirname(file)
        if not os.path.exists(dirname):
            os.makedirs(dirname)
    fid = open(file, "w").close()

def clean_tree(files):
    dirs = []
    for f in files:
        dirs.append(os.path.dirname(f))
        if os.path.exists(f):
            os.remove(f)

    for d in sorted(set(dirs))[::-1]:
        os.rmdir(d)

class TestStaticRepresentation(unittest.TestCase):
    def test_metadata(self):
        bento_info = """\
Name: Sphinx
Version: 0.6.3
Summary: Python documentation generator
Url: http://sphinx.pocoo.org/
DownloadUrl: http://pypi.python.org/pypi/Sphinx
Description: Some long description.
Author: Georg Brandl
AuthorEmail: georg@python.org
Maintainer: Georg Brandl
MaintainerEmail: georg@python.org
License: BSD
Platforms: any
Classifiers:
    Development Status :: 4 - Beta,
    Environment :: Console,
    Environment :: Web Environment,
    Intended Audience :: Developers,
    License :: OSI Approved :: BSD License,
    Operating System :: OS Independent,
    Programming Language :: Python,
    Topic :: Documentation,
    Topic :: Utilities
"""
        self._static_representation(bento_info)

    def test_simple_library(self):
        bento_info = """\
Name: foo

Library:
    Packages: foo
"""
        self._static_representation(bento_info)

    def _static_representation(self, bento_info):
        r_pkg = PackageDescription.from_string(bento_info)
        # We recompute pkg to avoid dealing with stylistic difference between
        # original and static_representation
        pkg = PackageDescription.from_string(static_representation(r_pkg))

        self.assertEqual(static_representation(pkg), static_representation(r_pkg))

class TestPackageMetadata(unittest.TestCase):
    def test_ctor(self):
        meta = PackageMetadata(name="foo", version="1.0", author="John Doe",
                               author_email="john@doe.com")
        self.assertEqual(meta.fullname, "foo-1.0")
        self.assertEqual(meta.contact, "John Doe")
        self.assertEqual(meta.contact_email, "john@doe.com")



import os
import sys
import marshal

# XXX: implementation details from py_compile
from py_compile import \
    wr_long, MAGIC, PyCompileError
import __builtin__

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

def bcompile(source):
    """Return the compiled bytecode from the given filename as a string ."""
    f = open(source, 'U')
    try:
        try:
            timestamp = long(os.fstat(f.fileno()).st_mtime)
        except AttributeError:
            timestamp = long(os.stat(file).st_mtime)
        codestring = f.read()
        f.close()
        if codestring and codestring[-1] != '\n':
            codestring = codestring + '\n'
        try:
            codeobject = __builtin__.compile(codestring, source, 'exec')
        except Exception,err:
            raise PyCompileError(err.__class__, err.args, source)
        fc = StringIO()
        try:
            fc.write('\0\0\0\0')
            wr_long(fc, timestamp)
            fc.write(marshal.dumps(codeobject))
            fc.flush()
            fc.seek(0, 0)
            fc.write(MAGIC)
            return fc.getvalue()
        finally:
            fc.close()
    finally:
        f.close()



import os

from bento.compat.api \
    import \
        NamedTemporaryFile
from bento.compat.api.moves \
    import \
        unittest
from bento.private.bytecode \
    import \
        bcompile, PyCompileError

def run_with_tempfile(content, function, mode="w"):
    """Create a temporary file with the given content, and execute tbe
    given function after the temporary file has been closed.

    The file is guaranteed to be deleted whether function succeeds or not
    """
    f = NamedTemporaryFile(mode=mode, delete=False)
    try:
        f.write(content)
        f.close()
        return function(f.name)
    finally:
        f.close()
        os.remove(f.name)

class TestBytecode(unittest.TestCase):
    def test_sanity(self):
        s = """print("foo")"""
        run_with_tempfile(s, lambda name: bcompile(name))

    def test_invalid(self):
        s = """print("""
        def f(filename):
            self.assertRaises(PyCompileError, lambda: bcompile(filename))
        run_with_tempfile(s, f)



from distutils.command.build \
    import \
        build as old_build

class build(old_build):
    cmd_name = "build"
    def __init__(self, *a, **kw):
        old_build.__init__(self, *a, **kw)

    def initialize_options(self):
        old_build.initialize_options(self)

    def finalize_options(self):
        old_build.finalize_options(self)

    def run(self):
        self.run_command("config")
        self.distribution.run_command_in_context(self.cmd_name, [])



import os
import shutil
import tempfile

import mock

import bento.core.testing

from bento.commands.command_contexts \
    import \
        ContextWithBuildDirectory
from bento.commands.options \
    import \
        OptionsContext
from bento.commands.sphinx_command \
    import \
        SphinxCommand
from bento.commands.wrapper_utils \
    import \
        run_command_in_context
from bento.compat.api.moves \
    import \
        unittest
from bento.core.package \
    import \
        PackageDescription
from bento.core.node \
    import \
        create_base_nodes

def has_sphinx():
    try:
        import sphinx
        return True
    except ImportError:
        return False

class TestSphinx(unittest.TestCase):
    def setUp(self):
        self.save = os.getcwd()
        self.d = tempfile.mkdtemp()
        os.chdir(self.d)
        try:
            self.top_node, self.build_node, self.run_node = \
                    create_base_nodes(self.d, os.path.join(self.d, "build"))
        except Exception:
            os.chdir(self.save)
            raise

    def tearDown(self):
        os.chdir(self.save)
        shutil.rmtree(self.d)

    @bento.core.testing.skip_if(not has_sphinx(), "sphinx not available, skipping sphinx command test(s)")
    def test_simple(self):
        n = self.top_node.make_node("doc/conf.py")
        n.parent.mkdir()
        n.write("")

        n = self.top_node.make_node("doc/contents.rst")
        n.write("")

        bento_info = "Name: foo"
        package = PackageDescription.from_string(bento_info)

        sphinx = SphinxCommand()
        opts = OptionsContext.from_command(sphinx)
        context = ContextWithBuildDirectory(None, [], opts, package, self.run_node)

        run_command_in_context(context, sphinx)



import sys

from distutils.command.sdist \
    import \
        sdist as old_sdist

class sdist(old_sdist):
    def __init__(self, *a, **kw):
        old_sdist.__init__(self, *a, **kw)

    def initialize_options(self):
        old_sdist.initialize_options(self)

    def finalize_options(self):
        old_sdist.finalize_options(self)

    def run(self):
        cmd_name = "sdist"
        cmd_argv = ["--output-dir=%s" % self.dist_dir]
        self.distribution.run_command_in_context(cmd_name, cmd_argv)



import os
import re
import subprocess
import _winreg

import yaku.task
from yaku.tools.mscommon.common \
    import \
        read_values, read_value, get_output

def _exec_command_factory(saved):
    def msvc_exec_command(self, cmd, cwd, env=None):
        new_cmd = []
        carry = ""
        for c in cmd:
            if c in ["/Fo", "/out:", "/OUT:", "/object:"]:
                carry = c
            else:
                c = carry + c
                carry = ""
                new_cmd.append(c)

        env = dict(os.environ)
        env.update(PATH=os.pathsep.join(self.env["PATH"]))
        saved(self, new_cmd, cwd, env=env)
    return msvc_exec_command

# Dict to 'canonalize' the arch
_ARCH_TO_CANONICAL = {
    "amd64"     : "amd64",
    "emt64"     : "amd64",
    "i386"      : "x86",
    "i486"      : "x86",
    "i586"      : "x86",
    "i686"      : "x86",
    "ia64"      : "ia64",
    "itanium"   : "ia64",
    "x86"       : "x86",
    "x86_64"    : "amd64",
}

# Given a (host, target) tuple, return the argument for the bat file. Both host
# and targets should be canonalized.
_HOST_TARGET_ARCH_TO_BAT_ARCH = {
    ("x86", "x86"): "x86",
    ("x86", "amd64"): "x86_amd64",
    ("amd64", "amd64"): "amd64",
    ("amd64", "x86"): "x86",
    ("x86", "ia64"): "x86_ia64"
}

_VCVER = ["10.0", "9.0", "9.0Exp","8.0", "8.0Exp","7.1", "7.0", "6.0"]

_VCVER_TO_PRODUCT_DIR = {
        '10.0': [
            r'Microsoft\VisualStudio\10.0\Setup\VC\ProductDir'],
        '9.0': [
            r'Microsoft\VisualStudio\9.0\Setup\VC\ProductDir'],
        '9.0Exp' : [
            r'Microsoft\VCExpress\9.0\Setup\VC\ProductDir'],
        '8.0': [
            r'Microsoft\VisualStudio\8.0\Setup\VC\ProductDir'],
        '8.0Exp': [
            r'Microsoft\VCExpress\8.0\Setup\VC\ProductDir'],
        '7.1': [
            r'Microsoft\VisualStudio\7.1\Setup\VC\ProductDir'],
        '7.0': [
            r'Microsoft\VisualStudio\7.0\Setup\VC\ProductDir'],
        '6.0': [
            r'Microsoft\VisualStudio\6.0\Setup\Microsoft Visual C++\ProductDir']
}

_is_win64 = None

def is_win64():
    """Return true if running on windows 64 bits.
    
    Works whether python itself runs in 64 bits or 32 bits."""
    # Unfortunately, python does not provide a useful way to determine
    # if the underlying Windows OS is 32-bit or 64-bit.  Worse, whether
    # the Python itself is 32-bit or 64-bit affects what it returns,
    # so nothing in sys.* or os.* help.  

    # Apparently the best solution is to use env vars that Windows
    # sets.  If PROCESSOR_ARCHITECTURE is not x86, then the python
    # process is running in 64 bit mode (on a 64-bit OS, 64-bit
    # hardware, obviously).
    # If this python is 32-bit but the OS is 64, Windows will set
    # ProgramW6432 and PROCESSOR_ARCHITEW6432 to non-null.
    # (Checking for HKLM\Software\Wow6432Node in the registry doesn't
    # work, because some 32-bit installers create it.)
    global _is_win64
    if _is_win64 is None:
        # I structured these tests to make it easy to add new ones or
        # add exceptions in the future, because this is a bit fragile.
        _is_win64 = False
        if os.environ.get('PROCESSOR_ARCHITECTURE','x86') != 'x86':
            _is_win64 = True
        if os.environ.get('PROCESSOR_ARCHITEW6432'):
            _is_win64 = True
        if os.environ.get('ProgramW6432'):
            _is_win64 = True
    return _is_win64


def msvc_version_to_maj_min(msvc_version):
   msvc_version_numeric = ''.join([x for  x in msvc_version if x in string_digits + '.'])

   t = msvc_version_numeric.split(".")
   if not len(t) == 2:
       raise ValueError("Unrecognized version %s (%s)" % (msvc_version,msvc_version_numeric))
   try:
       maj = int(t[0])
       min = int(t[1])
       return maj, min
   except ValueError, e:
       raise ValueError("Unrecognized version %s (%s)" % (msvc_version,msvc_version_numeric))

def is_host_target_supported(host_target, msvc_version):
    """Return True if the given (host, target) tuple is supported given the
    msvc version.

    Parameters
    ----------
    host_target: tuple
        tuple of (canonalized) host-target, e.g. ("x86", "amd64") for cross
        compilation from 32 bits windows to 64 bits.
    msvc_version: str
        msvc version (major.minor, e.g. 10.0)

    Note
    ----
    This only check whether a given version *may* support the given (host,
    target), not that the toolchain is actually present on the machine.
    """
    # We assume that any Visual Studio version supports x86 as a target
    if host_target[1] != "x86":
        maj, min = msvc_version_to_maj_min(msvc_version)
        if maj < 8:
            return False

    return True

def find_vc_pdir(msvc_version):
    """Try to find the product directory for the given
    version.

    Note
    ----
    If for some reason the requested version could not be found, an
    exception which inherits from VisualCException will be raised."""
    base = _winreg.HKEY_LOCAL_MACHINE
    root = 'Software\\'
    if is_win64():
        root = root + 'Wow6432Node\\'
    try:
        hkeys = _VCVER_TO_PRODUCT_DIR[msvc_version]
    except KeyError:
        #debug("Unknown version of MSVC: %s" % msvc_version)
        raise ValueError("Unknown version %s" % msvc_version)

    for key in hkeys:
        key = root + key
        comps = read_value(key)
        if comps is not None:
            if os.path.exists(comps):
                return comps
            else:
                raise ValueError("registry dir %s not found on the filesystem" % comps)
    return None

def find_versions(abi):
    base = _winreg.HKEY_LOCAL_MACHINE
    key = os.path.join(_FC_ROOT[abi], "Fortran")

    availables = {}
    versions = read_keys(base, key)
    if versions is None:
        return availables
    for v in versions:
        verk = os.path.join(key, v)
        key = open_key(verk)
        try:
            maj = _winreg.QueryValueEx(key, "Major Version")[0]
            min = _winreg.QueryValueEx(key, "Minor Version")[0]
            bld = _winreg.QueryValueEx(key, "Revision")[0]
            availables[(maj, min, bld)] = verk
        finally:
            close_key(key)
    return availables

def _detect_msvc(ctx):
    from string import digits as string_digits

    msvc_version_info = (9, 0)
    msvc_version = "9.0"
    pdir = find_vc_pdir(msvc_version)
    if pdir is None:
        raise ValueError("VS 9.0 not found")

    # filter out e.g. "Exp" from the version name
    msvc_ver_numeric = ''.join([x for x in msvc_version if x in string_digits + "."])
    vernum = float(msvc_ver_numeric)
    if 7 <= vernum < 8:
        pdir = os.path.join(pdir, os.pardir, "Common7", "Tools")
        batfilename = os.path.join(pdir, "vsvars32.bat")
    elif vernum < 7:
        pdir = os.path.join(pdir, "Bin")
        batfilename = os.path.join(pdir, "vcvars32.bat")
    else: # >= 8
        batfilename = os.path.join(pdir, "vcvarsall.bat")

    vc_paths = get_output(ctx, batfilename, "x86")
    cc = None
    linker = None
    lib = None
    for p in vc_paths["PATH"]:
        _cc = os.path.join(p, "cl.exe")
        _linker = os.path.join(p, "link.exe")
        _lib = os.path.join(p, "lib.exe")
        if os.path.exists(_cc) and os.path.exists(_linker) and os.path.exists(_lib):
            cc = _cc
            linker = _linker
            lib = _lib
            break
    if cc is None or linker is None:
        raise RuntimeError("Could not find cl.exe/link.exe")

    return cc, linker, lib, vc_paths, msvc_version_info

def setup(ctx):
    env = ctx.env

    cc, linker, lib, vc_paths, msvc_version_info = _detect_msvc(ctx)
    ctx.env["PATH"] = vc_paths["PATH"][:]
    ctx.env.prextend("CPPPATH", vc_paths["INCLUDE"], create=True)
    ctx.env.prextend("LIBDIR", vc_paths["LIB"], create=True)

    ctx.env["CC"] = [cc]
    ctx.env["CC_TGT_F"] = ["/c", "/Fo"]
    ctx.env["CC_SRC_F"] = []
    ctx.env["CFLAGS"] = ["/nologo"]
    ctx.env["CPPPATH_FMT"] = "/I%s"
    ctx.env["DEFINES"] = []
    ctx.env["DEFINES_FMT"] = "/D%s"
    ctx.env["LINK"] = [linker]
    ctx.env["LINK_TGT_F"] = ["/out:"]
    ctx.env["LINK_SRC_F"] = []
    ctx.env["LINKFLAGS"] = ["/nologo"]
    ctx.env["SHLINK"] = [linker, "/DLL"]
    ctx.env["SHLINK_TGT_F"] = ["/out:"]
    ctx.env["SHLINK_SRC_F"] = []
    ctx.env["SHLINKFLAGS"] = []
    ctx.env["MODLINK"] = [linker, "/DLL"]
    ctx.env["MODLINK_TGT_F"] = ["/out:"]
    ctx.env["MODLINK_SRC_F"] = []
    ctx.env["MODLINKFLAGS"] = ["/nologo"]
    ctx.env["LIBS"] = []
    ctx.env["LIB_FMT"] = "%s.lib"
    ctx.env["LIBDIR"] = []
    ctx.env["LIBDIR_FMT"] = "/LIBPATH:%s"

    ctx.env["STLINK"] = [lib]
    ctx.env["STLINK_TGT_F"] = ["/OUT:"]
    ctx.env["STLINK_SRC_F"] = []
    ctx.env["STLINKFLAGS"] = ["/nologo"]
    ctx.env["STATICLIB_FMT"] = "%s.lib"

    ctx.env["CXX"] = [cc]
    ctx.env["CXX_TGT_F"] = ["/c", "/Fo"]
    ctx.env["CXX_SRC_F"] = []
    ctx.env["CXXFLAGS"] = ["/nologo"]
    if msvc_version_info >= (9, 0):
        ctx.env.append("CXXFLAGS", "/EHsc")
    ctx.env["CXXLINK"] = [linker]
    ctx.env["CXXLINKFLAGS"] = ["/nologo"]
    ctx.env["CXXLINK_TGT_F"] = ["/out:"]
    ctx.env["CXXLINK_SRC_F"] = []
    ctx.env["CXXSHLINK"] = [linker]
    ctx.env["CXXSHLINKFLAGS"] = []
    ctx.env["CXXSHLINK_TGT_F"] = ["/out:"]
    ctx.env["CXXSHLINK_SRC_F"] = []

    ctx.env["CC_OBJECT_FMT"] = "%s.obj"
    ctx.env["CXX_OBJECT_FMT"] = "%s.obj"
    ctx.env["SHAREDLIB_FMT"] = "%s.dll"
    ctx.env["PROGRAM_FMT"] = "%s.exe"

    for k, v in vc_paths.items():
        k = k.encode("ascii")
        if k in ["LIB"]:
            env.extend("LIBDIR", v, create=True)
        elif k in ["CPPPATH"]:
            env.extend(k, v, create=True)

for task_class in ["cc", "shcc", "cc_shlink", "cc_stlink", "cc_program", "cxx", "cxxprogram", "pycc", "pycxx", "pylink"]:
    klass = yaku.task.task_factory(task_class)
    saved = klass.exec_command
    klass.exec_command = _exec_command_factory(saved)

def detect(ctx):
    _detect_msvc(ctx)
    return True



import os

from bento.compat.api \
    import \
        relpath

from bento.commands.hooks \
    import \
        create_hook_module

def run_with_dependencies(global_context, cmd_name, cmd_argv, run_node, top_node, package):
    """Run the given command, including its dependencies as defined in the
    global_context."""
    deps = global_context.retrieve_dependencies(cmd_name)
    for dep_cmd_name in deps:
        dep_cmd_argv = global_context.retrieve_command_argv(dep_cmd_name)
        resolve_and_run_command(global_context, dep_cmd_name, dep_cmd_argv, run_node, top_node, package)
    resolve_and_run_command(global_context, cmd_name, cmd_argv, run_node, top_node, package)

def resolve_and_run_command(global_context, cmd_name, cmd_argv, run_node, top_node, package):
    """Run the given Command instance inside its context, including any hook
    and/or override."""
    cmd = global_context.retrieve_command(cmd_name)
    context_klass = global_context.retrieve_command_context(cmd_name)
    options_context = global_context.retrieve_options_context(cmd_name)

    context = context_klass(global_context, cmd_argv, options_context, package, run_node)

    pre_hooks = global_context.retrieve_pre_hooks(cmd_name)
    post_hooks = global_context.retrieve_post_hooks(cmd_name)

    run_command_in_context(context, cmd, pre_hooks, post_hooks)

    return cmd, context

def run_command_in_context(context, cmd, pre_hooks=None, post_hooks=None):
    """Run the given command instance with the hooks within its context. """
    if pre_hooks is None:
        pre_hooks = []
    if post_hooks is None:
        post_hooks = []

    top_node = context.top_node
    cmd_funcs = [(cmd.run, top_node.abspath())]

    def _run_hooks(hooks):
        for hook in hooks:
            local_node = top_node.find_dir(relpath(hook.local_dir, top_node.abspath()))
            context.pre_recurse(local_node)
            try:
                if not context.help:
                    hook(context)
            finally:
                context.post_recurse()

    context.init()
    try:
        cmd.init(context)

        _run_hooks(pre_hooks)

        context.configure()

        while cmd_funcs:
            cmd_func, local_dir = cmd_funcs.pop(0)
            local_node = top_node.find_dir(relpath(local_dir, top_node.abspath()))
            context.pre_recurse(local_node)
            try:
                cmd_func(context)
            finally:
                context.post_recurse()

        _run_hooks(post_hooks)

        cmd.finish(context)
    finally:
        context.finish()

    return cmd, context

def set_main(pkg, top_node, build_node):
    modules = []
    hook_files = pkg.hook_files
    for name, spkg in pkg.subpackages.items():
        hook_files.extend([os.path.join(spkg.rdir, h) for h in spkg.hook_files])

    # TODO: find doublons
    for f in hook_files:
        hook_node = top_node.make_node(f)
        if hook_node is None or not os.path.exists(hook_node.abspath()):
            raise ValueError("Hook file %s not found" % f)
        modules.append(create_hook_module(hook_node.abspath()))
    return modules



import os
import copy
import shutil
import tempfile

import os.path as op

import bento.convert.commands

from bento.errors \
    import \
        UsageException
from bento.compat.api.moves \
    import \
        unittest
from bento.core.package \
    import \
        PackageDescription
from bento.core.node \
    import \
        create_root_with_source_tree
from bento.core.testing \
    import \
        create_fake_package_from_bento_info
from bento.commands.command_contexts \
    import \
        CmdContext
from bento.commands.options \
    import \
        OptionsContext
from bento.convert.commands \
    import \
        ConvertCommand, DetectTypeCommand
from bento.testing.sub_test_case \
    import \
        SubprocessTestCase

dummy_meta_data = dict(
        name="foo",
        version="1.0",
        description="a few words",
        long_description="some more words",
        url="http://example.com",
        download_url="http://example.com/download",
        author="John Doe",
        maintainer="John Doe",
        author_email="john@example.com",
        maintainer_email="john@example.com",
        license="BSD",
        platforms=["UNIX"],
)

bento_dummy_meta_data = copy.copy(dummy_meta_data)
bento_dummy_meta_data["platforms"] = ",".join(bento_dummy_meta_data["platforms"])

bento_meta_data_template = """\
Name: %(name)s
Version: %(version)s
Summary: %(description)s
Url: %(url)s
DownloadUrl: %(download_url)s
Description: %(long_description)s
Author: %(author)s
AuthorEmail: %(author_email)s
Maintainer: %(maintainer)s
MaintainerEmail: %(maintainer_email)s
License: %(license)s
Platforms: %(platforms)s"""

def _run_convert_command(top_node, run_node, setup_py, bento_info, cmd_argv):
    setup_node = top_node.make_node("setup.py")
    setup_node.safe_write(setup_py)

    create_fake_package_from_bento_info(top_node, bento_info)
    package = PackageDescription.from_string(bento_info)

    cmd = ConvertCommand()
    opts = OptionsContext.from_command(cmd)

    context = CmdContext(None, cmd_argv, opts, package, run_node)
    cmd.run(context)
    cmd.finish(context)
    context.finish()

class CommonTestCase(unittest.TestCase):
    def setUp(self):
        super(CommonTestCase, self).setUp()
        self.save = os.getcwd()
        self.d = tempfile.mkdtemp()
        os.chdir(self.d)
        try:
            self.root = create_root_with_source_tree(self.d, os.path.join(self.d, "build"))
            self.top_node = self.root._ctx.srcnode
            self.build_node = self.root._ctx.bldnode
            self.run_node = self.root.find_node(self.d)
        except Exception:
            os.chdir(self.save)
            raise

    def tearDown(self):
        os.chdir(self.save)
        shutil.rmtree(self.d)

        super(CommonTestCase, self).tearDown()

class TestConvertCommand(SubprocessTestCase, CommonTestCase):
    def test_simple_package(self):
        bento_meta_data = bento_meta_data_template % bento_dummy_meta_data
        bento_info = """\
%s

ExtraSourceFiles:
    setup.py

Library:
    Packages:
        foo
""" % bento_meta_data

        setup_py = """\
from distutils.core import setup
setup(packages=["foo"], **%s)
""" % dummy_meta_data

        output = "foo.info"
        cmd_argv = ["--output=%s" % output, "-t", "distutils"]

        _run_convert_command(self.top_node, self.run_node, setup_py, bento_info, cmd_argv=cmd_argv)
        gen_bento = self.top_node.find_node(output)
        self.assertEqual(gen_bento.read(), bento_info)

    def test_package_data_distutils(self):
        bento_meta_data = bento_meta_data_template % bento_dummy_meta_data
        bento_info = """\
%s

ExtraSourceFiles:
    setup.py

DataFiles: foo_data
    SourceDir: foo
    TargetDir: $sitedir/foo
    Files:
        info.txt

Library:
    Packages:
        foo
""" % bento_meta_data

        setup_py = """\
from distutils.core import setup
setup(packages=["foo"], package_data={"foo": ["*txt"]}, **%s)
""" % dummy_meta_data

        data_node = self.top_node.make_node(op.join("foo", "info.txt"))
        data_node.parent.mkdir()
        data_node.write("")

        output = "foo.info"
        cmd_argv = ["--output=%s" % output, "-t", "distutils"]

        _run_convert_command(self.top_node, self.run_node, setup_py, bento_info, cmd_argv=cmd_argv)
        gen_bento = self.top_node.find_node(output)
        self.assertEqual(gen_bento.read(), bento_info)

class TestMockedConvertCommand(CommonTestCase):
    """Test the convert command UI."""
    def setUp(self):
        super(TestMockedConvertCommand, self).setUp()

        def dummy_convert(ctx, filename, setup_args, monkey_patch_mode, verbose, output, log, show_output):
            pass
        self.old_convert = bento.convert.commands.convert
        try:
            bento.convert.commands.convert = lambda *a: None
        except:
            bento.convert.commands.convert = self.old_convert

    def tearDown(self):
        bento.convert.commands.convert.convert = self.old_convert

        super(TestMockedConvertCommand, self).tearDown()

    def test_simple(self):
        bento_info = """\
Name: foo

Library:
    Packages:
        foo
"""

        setup_py = """\
from distutils.core import setup
setup(packages=["foo"], name="foo")
"""
        _run_convert_command(self.top_node, self.run_node, setup_py, bento_info, [])

    def test_help(self):
        bento_info = """\
Name: foo

Library:
    Packages:
        foo
"""

        setup_py = """\
from distutils.core import setup
setup(packages=["foo"], name="foo")
"""
        _run_convert_command(self.top_node, self.run_node, setup_py, bento_info, ["-h"])

    def test_not_overwritten(self):
        bento_info = """\
Name: foo

Library:
    Packages:
        foo
"""

        setup_py = """\
from distutils.core import setup
setup(packages=["foo"], name="foo")
"""
        self.top_node.make_node("bento.info").write("")
        self.assertRaises(UsageException,
                          lambda: _run_convert_command(self.top_node,
                              self.run_node, setup_py, bento_info,
                              ["--output=bento.info"]))

class TestMockedDetectTypeCommand(CommonTestCase):
    """Test the detect_type command UI."""
    def setUp(self):
        super(TestMockedDetectTypeCommand, self).setUp()

        self.old_whole_test = bento.convert.commands.whole_test
        try:
            bento.convert.commands.whole_test = lambda *a: None
        except:
            bento.convert.commands.whole_test = self.old_whole_test

    def tearDown(self):
        bento.convert.commands.convert.whole_test = self.old_whole_test

        super(TestMockedDetectTypeCommand, self).tearDown()

    def _run_command(self):
        setup_node = self.top_node.make_node("setup.py")
        setup_node.safe_write("")

        create_fake_package_from_bento_info(self.top_node, "")
        package = PackageDescription.from_string("")

        cmd = DetectTypeCommand()
        opts = OptionsContext.from_command(cmd)

        context = CmdContext(None, [], opts, package, self.run_node)
        cmd.run(context)
        cmd.finish(context)
        context.finish()

    def test_simple(self):
        self._run_command()



import os

from yaku.environment \
    import \
        Environment

RULES_REGISTRY = {}
FILES_REGISTRY = {}

def extension(ext):
    def _f(f):
        RULES_REGISTRY[ext] = f
        return f
    return _f

def set_extension_hook(ext, hook):
    old = RULES_REGISTRY.get(ext, None)
    RULES_REGISTRY[ext] = hook
    return old

def wrap_extension_hook(ext, hook_factory):
    old = RULES_REGISTRY.get(ext, None)
    RULES_REGISTRY[ext] = hook_factory(old)
    return old

def get_extension_hook(ext):
    try:
        return RULES_REGISTRY[ext]
    except KeyError:
        raise ValueError("No hook registered for extension %r" % ext)

def set_file_hook(ctx, source, hook):
    node = ctx.src_root.find_resource(source)
    if node is None:
        raise IOError("file %s not found" % source)
    FILES_REGISTRY[node] = hook

class NoHookException(Exception):
    pass

def hash_task(t):
    # FIXME: ext_in and ext_out should not be computed from the files
    ext_in = [os.path.splitext(s.name)[1] for s in t.inputs]
    ext_out = [os.path.splitext(s.name)[1] for s in t.outputs]
    tup = tuple(ext_in + ext_out + t.before + t.after)
    return hash((t.__class__.__name__, tup))

class TaskManager(object):
    def __init__(self, tasks):
        self.tasks = tasks

        self.groups = {}
        self.order = {}
        self.make_groups()
        self.make_order()

    def set_order(self, a, b):
        if not a in self.order:
            self.order[a] = set()
        self.order[a].add(b)

    def make_order(self):
        keys = list(self.groups.keys())
        max = len(keys)
        for i in range(max):
            t1 = self.groups[keys[i]][0]
            for j in range(i + 1, max):
                t2 = self.groups[keys[j]][0]

                if t2.__class__.__name__ in t1.before:
                    self.set_order(keys[j], keys[i])
                elif t1.__class__.__name__ in t2.before:
                    self.set_order(keys[i], keys[j])
                else:
                    # add the constraints based on the comparisons
                    val = self.compare_exts(t1, t2)
                    if val > 0:
                        self.set_order(keys[i], keys[j])
                    elif val < 0:
                        self.set_order(keys[j], keys[i])

    def make_groups(self):
        # XXX: we assume tasks with same input/output suffix can run
        # in // (naive emulation of csr-like scheduler in waf)
        groups = self.groups
        for t in self.tasks:
            h = hash_task(t)
            if h in groups:
                groups[h].append(t)
            else:
                groups[h] = [t]

    def next_set(self):
        keys = self.groups.keys()

        unconnected = []
        remainder = []

        for u in keys:
            for k in self.order.values():
                if u in k:
                    remainder.append(u)
                    break
            else:
                unconnected.append(u)

        toreturn = []
        for y in unconnected:
            toreturn.extend(self.groups[y])

        # remove stuff only after
        for y in unconnected:
                try:
                    self.order.__delitem__(y)
                except KeyError:
                    pass
                self.groups.__delitem__(y)

        if not toreturn and remainder:
            raise Exception("circular order constraint detected %r" % remainder)

        return toreturn


    def compare_exts(self, t1, t2):
        "extension production"
        def _get_in(t):
            return [os.path.splitext(s.name)[1] for s in t.inputs]
        def _get_out(t):
            return [os.path.splitext(s.name)[1] for s in t.outputs]

        in_ = _get_in(t1)
        out_ = _get_out(t2)
        for k in in_:
            if k in out_:
                return -1
        in_ = _get_in(t2)
        out_ = _get_out(t1)
        for k in in_:
            if k in out_:
                return 1
        return 0

def run_task(ctx, task):
    def _run(t):
        t.run()
        ctx.cache[tuid] = t.signature()

    tuid = task.get_uid()
    # XXX: there may be a better way to do this without stating output
    # (we want to know if the task has already been executed in a
    # previous run)
    for o in task.outputs:
        if not os.path.exists(o.abspath()):
            _run(task)
            break
    if not tuid in ctx.cache:
        _run(task)
    else:
        sig = task.signature()
        if sig != ctx.cache[tuid]:
            _run(task)

def build_dag(tasks):
    # Build dependency graph (DAG)
    # task_deps[target] = list_of_dependencies
    # At this point, task_deps is not guaranteed to be a DAG (may have
    # cycle) - will be detected during topological sort
    task_deps = {}
    output_to_tuid = {}
    for t in tasks:
        for o in t.outputs:
            try:
                task_deps[o].extend(t.inputs + t.deps)
            except KeyError:
                task_deps[o] = t.inputs[:] + t.deps[:]
            output_to_tuid[o] = t.get_uid()
    return task_deps, output_to_tuid

def topo_sort(task_deps):
    # Topological sort (depth-first search)
    # XXX: cycle detection is missing
    tmp = []
    nodes = []
    for dep in task_deps.values():
        nodes.extend(dep)
    nodes.extend(task_deps.keys())
    nodes = set(nodes)

    visited = set()
    def visit(node):
        if not node in visited:
           visited.add(node)
           deps = task_deps.get(node, None)
           if deps:
               for c in deps:
                   visit(c)
           tmp.append(node)

    for node in nodes:
        visit(node)

    return tmp

def _get_hook(source):
    if source in FILES_REGISTRY:
        return FILES_REGISTRY[source]
    for ext in RULES_REGISTRY:
        if source.name.endswith(ext):
            return RULES_REGISTRY[ext]
    raise NoHookException(
            "No rule defined for extension %r" % source.suffix())

class TaskGen(object):
    def __init__(self, name, bld, sources, target):
        self.bld = bld
        self.name = name
        self.sources = sources
        self.target = target

        self.env = Environment()

    def process(self):
        tasks = []
        for s in self.sources:
            f = _get_hook(s)
            tsks = f(self, s)
            tasks.extend(tsks)
        return tasks

class CompiledTaskGen(TaskGen):
    def __init__(self, name, bld, sources, target):
        TaskGen.__init__(self, name, bld, sources, target)
        self.object_tasks = []
        self.link_task = None
        self.has_cxx = False

    def add_objects(self, tasks):
        """Add new object tasks, assuming the link task has already
        been defined."""
        if self.link_task is None:
            raise ValueError("You cannot add object nodes "
                             "before setting the link task !")
        else:
            self.object_tasks += tasks

def order_tasks(tasks):
    tuid_to_task = dict([(t.get_uid(), t) for t in tasks])

    task_deps, output_to_tuid = build_dag(tasks)

    yo = topo_sort(task_deps)
    ordered_tasks = []
    for output in yo:
        if output in output_to_tuid:
            ordered_tasks.append(tuid_to_task[output_to_tuid[output]])

    return ordered_tasks




import os
import sys
import copy
import warnings

from bento.compat.api import json
import bento.compat.api as compat

from bento.core.node \
    import \
        find_root
from bento.core.platforms \
    import \
        get_scheme
from bento.utils.utils import subst_vars, same_content, fix_kw, explode_path
from bento.core.pkg_objects \
    import \
        Executable
import bento.utils.path

def ipkg_meta_from_pkg(pkg):
    """Return meta dict for Installed pkg from a PackageDescription
    instance."""
    meta = {}
    for m in ["name", "version", "summary", "url", "author",
              "author_email", "license", "download_url", "description",
              "platforms", "classifiers", "install_requires", 
              "top_levels"]:
        meta[m] = getattr(pkg, m)
    return meta

class InstalledSection(object):
    @classmethod
    def from_source_target_directories(cls, category, name, source_dir, target_dir, files):
        files = [(f, f) for f in files]
        return cls(category, name, source_dir, target_dir, files)

    def __init__(self, category, name, srcdir, target, files):
        self.category = category
        self.name = name
        if os.sep != "/":
            self.source_dir = bento.utils.path.normalize_path(srcdir)
            self.target_dir = bento.utils.path.normalize_path(target)
            self.files = [(bento.utils.path.normalize_path(f), bento.utils.path.normalize_path(g)) for f, g in files]
        else:
            self.source_dir = srcdir
            self.target_dir = target
            self.files = files

    def __repr__(self):
        ret = """\
InstalledSection(%(category)s,
    %(name)s,
    %(source_dir)s,
    %(target_dir)s,
    %(files)s)""" % {"name": self.name, "category": self.category,
            "source_dir": self.source_dir, "target_dir": self.target_dir,
            "files": self.files}
        return ret

    def __eq__(self, other):
        return self.category == other.category and \
                self.name == other.name and \
                self.source_dir == other.source_dir and \
                self.target_dir == other.target_dir and \
                self.files == other.files

def iter_source_files(file_sections):
    for kind in file_sections:
        if not kind in ["executables"]:
            for name, section in file_sections[kind].items():
                for f in section:
                    yield f[0]

def iter_files(file_sections):
    # XXX: what to do with multiple source for a same target ? It is not always
    # easy to avoid this situation, especially for python files and files
    # installed from wildcards. For now, we raise an exception unless the
    # sources have the exact same content, but this may not be enough (what if
    # category changes ? This may cause different target permissions and other
    # category-specific post-processing during install)
    installed_files = {}
    def _is_redundant(source, target):
        source_path = source.abspath()
        target_path = target.abspath()
        # If this install already installs something @ target, we raise an
        # error unless the content is exactly the same
        if not target in installed_files:
            installed_files[target] = source
            return False
        else:
            if not same_content(source_path, installed_files[target].abspath()):
                # See top comment: not sure there is any good solution to
                # select which one should be selected when a target has
                # multiple sources
                if source.is_src() and installed_files[target].is_bld():
                    return False
                if source.is_bld() and installed_files[target].is_src():
                    return True
                else:
                    raise IOError("Multiple source_path for same target_path %r ! %s and %s" % \
                                  (target_path, source_path, installed_files[target].abspath()))
            else:
                return True

    for kind in file_sections:
        for name, section in file_sections[kind].items():
            for source, target in section:
                if not _is_redundant(source, target):
                    yield kind, source, target

class InstalledPkgDescription(object):
    @classmethod
    def from_egg(cls, egg_path):
        zid = compat.ZipFile(egg_path)
        try:
            data = json.loads(zid.read("EGG-INFO/ipkg.info"))
            return cls.from_json_dict(data)
        finally:
            zid.close()

    @classmethod
    def from_json_dict(cls, d):
        return cls.__from_data(d)

    @classmethod
    def from_string(cls, s):
        return cls.__from_data(json.loads(s))

    @classmethod
    def from_file(cls, filename):
        fid = open(filename)
        try:
            return cls.__from_data(json.load(fid))
        finally:
            fid.close()

    @classmethod
    def __from_data(cls, data):
        meta_vars = fix_kw(data["meta"])
        #variables = data["variables"]
        install_paths = data.get("install_paths", None)

        executables = {}
        for name, executable in data["executables"].items():
            executables[name] = Executable.from_parse_dict(fix_kw(executable))

        file_sections = {}

        def json_to_file_section(data):
            category = data["category"]
            name = data["name"]
            section = InstalledSection(category, name, data["source_dir"],
                                       data["target_dir"], data["files"])
            return category, name, section

        for section in data["file_sections"]:
            category, name, files = json_to_file_section(section)
            if category in file_sections:
                if name in file_sections[category]:
                    raise ValueError("section %s of type %s already exists !" % (name, category))
                file_sections[category][name] = files
            else:
                file_sections[category] = {name: files}
        return cls(file_sections, meta_vars, executables, install_paths)

    def __init__(self, files, meta, executables, path_variables=None):
        self.files = files
        self.meta = meta
        if path_variables is None:
            self._path_variables = get_scheme(sys.platform)[0]
        else:
            self._path_variables = path_variables
        self.executables = executables

        self._variables = {"pkgname": self.meta["name"],
                           "py_version_short": ".".join([str(i) for i in sys.version_info[:2]])}

    def write(self, filename):
        fid = open(filename, "w")
        try:
            return self._write(fid)
        finally:
            fid.close()

    def _write(self, fid):
        def executable_to_json(executable):
            return {"name": executable.name,
                    "module": executable.module,
                    "function": executable.function}

        def section_to_json(section):
            return {"name": section.name,
                    "category": section.category,
                    "source_dir": section.source_dir,
                    "target_dir": section.target_dir,
                    "files": section.files}

        data = {}
        data["meta"] = self.meta

        executables = dict([(k, executable_to_json(v)) \
                            for k, v in self.executables.items()])
        data["executables"] = executables
        data["install_paths"] = self._path_variables

        file_sections = []
        for category, value in self.files.items():
            if category in ["pythonfiles", "bentofiles"]:
                for i in value.values():
                    i.srcdir = "$_srcrootdir"
                    file_sections.append(section_to_json(i))
            elif category in ["datafiles", "extensions", "executables",
                        "compiled_libraries"]:
                for i in value.values():
                    file_sections.append(section_to_json(i))
            else:
                warnings.warn("Unknown category %r" % category)
                for i in value.values():
                    file_sections.append(section_to_json(i))
        data["file_sections"] = file_sections
        if "BENTOMAKER_PRETTY" in os.environ:
            json.dump(data, fid, sort_keys=True, indent=4)
        else:
            json.dump(data, fid, separators=(',', ':'))

    def update_paths(self, paths):
        for k, v in paths.items():
            self._path_variables[k] = v

    def resolve_path(self, path):
        variables = copy.copy(self._path_variables)
        variables.update(self._variables)
        return subst_vars(path, variables)

    def resolve_paths_with_destdir(self, src_root_node):
        """Same as resolve_paths, but prefix every path with $destdir."""
        return self._resolve_paths(src_root_node, use_destdir=True)

    def resolve_paths(self, src_root_node):
        return self._resolve_paths(src_root_node, use_destdir=False)

    def _resolve_paths(self, src_root_node, use_destdir):
        variables = copy.copy(self._path_variables)
        variables.update(self._variables)
        variables['_srcrootdir'] = src_root_node.abspath()

        root = find_root(src_root_node)

        def _prefix_destdir(path):
            destdir = subst_vars("$destdir", variables)
            if path:
                tail = explode_path(path)[1:]
                if not tail:
                    raise ValueError("Invalid target directory in section %r "
                                     "(not absolute: %r)" % (name, path))
                return os.path.join(destdir, os.path.join(*tail))
            else:
                raise ValueError("Invalid target directory in section "
                                 "%r: %r" % (name, path))

        node_sections = {}
        for category in self.files:
            node_sections[category] = {}
            for name, section in self.files[category].items():
                srcdir = subst_vars(section.source_dir, variables)
                target = subst_vars(section.target_dir, variables)

                if use_destdir:
                    target = _prefix_destdir(target)

                srcdir_node = root.find_node(srcdir)
                if srcdir_node is None:
                    raise IOError("directory %r not found !" % (srcdir,))
                target_node = root.make_node(target)
                node_sections[category][name] = \
                        [(srcdir_node.find_node(f), target_node.make_node(g))
                         for f, g in section.files]

        return node_sections



import os.path as op

import bento.errors

from bento.commands.core \
    import \
        Command, Option
from bento.pypi.register_utils \
    import \
        build_post_data, post_to_server, DEFAULT_REPOSITORY, read_pypirc, PyPIConfig

from six \
    import \
        PY3

if PY3:
    from urllib.request \
        import \
            HTTPPasswordMgr, urlparse
else:
    from urllib2 \
        import \
            HTTPPasswordMgr
    from urlparse \
        import \
            urlparse

def _read_pypirc(repository):
    filename = op.join(op.expanduser("~"), ".pypirc")
    if not op.exists(filename):
        raise bento.errors.UsageException(
                "file %r not found (automatic creation of .pypirc not supported yet" % filename)
    try:
        return read_pypirc(repository)
    except bento.errors.InvalidPyPIConfig:
        raise bento.errors.UsageException("repository %r not found in %r" % (repository, filename))

class RegisterPyPI(Command):
    long_descr = """\
Purpose: register the package to pypi
Usage: bentomaker register [OPTIONS]"""
    short_descr = "register packages to pypi."
    common_options = Command.common_options \
                        + [Option("-r", "--repository",
                                  help="Repository to use in .pypirc"),
                           Option("-u", "--username",
                                  help="Username to use for registration"),
                           Option("-p", "--password",
                                  help="Password to use for registration"),
                           Option( "--repository-url",
                                  help="Repository URL to use for registration"),
                                  ]
    def run(self, context):
        o, a = context.get_parsed_arguments()
        if o.repository and (o.username or o.password or o.repository_url):
            raise bento.errors.UsageException(
                    "Cannot specify repository and username/password/url at the same time")
        if not (o.repository or (o.username or o.password or o.repository_url)):
            # FIXME: why does distutils use DEFAULT_REPOSITORY (i.e. an url)
            # here ?
            config = _read_pypirc(DEFAULT_REPOSITORY)
        elif o.repository:
            config = _read_pypirc(o.repository)
        else:
            config = PyPIConfig(o.username, o.password, o.repository_url)

        auth = HTTPPasswordMgr()
        host = urlparse(config.repository)[0]
        auth.add_password(config.realm, host, config.username, config.password)

        post_data = build_post_data(context.pkg, "submit")
        code, msg = post_to_server(post_data, config)
        if code != 200:
            raise bento.errors.BentoError("Error while submitting package metadata to server: %r" % msg)



import sys
import traceback
if sys.version_info[0] < 3:
    import Queue as queue
else:
    import queue
import threading

from yaku.task_manager \
    import \
        run_task, order_tasks, TaskManager
from yaku.utils \
    import \
        get_exception
import yaku.errors

def run_tasks(ctx, tasks=None):
    if tasks is None:
        tasks = ctx.tasks
    task_manager = TaskManager(tasks)
    s = SerialRunner(ctx, task_manager)
    s.start()
    s.run()

def run_tasks_parallel(ctx, tasks=None, maxjobs=1):
    if tasks is None:
        tasks = ctx.tasks
    task_manager = TaskManager(tasks)
    r = ParallelRunner(ctx, task_manager, maxjobs)
    r.start()
    r.run()

class SerialRunner(object):
    def __init__(self, ctx, task_manager):
        self.ctx = ctx
        self.task_manager = task_manager

    def start(self):
        # Dummy to give same interface as ParallelRunner
        pass

    def run(self):
        grp = self.task_manager.next_set()
        while grp:
            for task in grp:
                run_task(self.ctx, task)
            grp = self.task_manager.next_set()

class ParallelRunner(object):
    def __init__(self, ctx, task_manager, maxjobs=1):
        self.njobs = maxjobs
        self.task_manager = task_manager
        self.ctx = ctx

        self.worker_queue = queue.Queue()
        self.error_out = queue.Queue()
        self.failure_lock = threading.Lock()
        self.stop = False

    def start(self):
        def _worker():
            # XXX: this whole thing is an hack - find a better way to
            # notify task execution failure to all worker threads
            while not self.stop:
                task = self.worker_queue.get()
                try:
                    run_task(self.ctx, task)
                except yaku.errors.TaskRunFailure:
                    e = get_exception()
                    self.failure_lock.acquire()
                    self.stop = True
                    self.failure_lock.release()
                    task.error_msg = e.explain
                    task.error_cmd = e.cmd
                    self.error_out.put(task)
                except Exception:
                    e = get_exception()
                    exc_type, exc_value, tb = sys.exc_info()
                    lines = traceback.format_exception(exc_type, exc_value, tb)
                    self.failure_lock.acquire()
                    self.stop = True
                    self.failure_lock.release()
                    task.error_msg = "".join(lines)
                    task.error_cmd = []
                    self.error_out.put(task)
                self.worker_queue.task_done()

        for i in range(self.njobs):
            t = threading.Thread(target=_worker)
            t.setDaemon(True)
            t.start()

    def run(self):
        grp = self.task_manager.next_set()
        while grp:
            for task in grp:
                self.worker_queue.put(task)
            # XXX: we only join once we detect the worker queue to be empty, to
            # avoid blocking for a long time. This is naive, and will break if
            # the worker_queue is filled after this point
            while not self.stop:
                if self.worker_queue.empty():
                    self.worker_queue.join()
                    break
            if not self.error_out.empty():
                task = self.error_out.get()
                msg = task.error_msg
                cmd = task.error_cmd
                raise yaku.errors.TaskRunFailure(cmd, msg)

            grp = self.task_manager.next_set()



import os.path as op

from bento.distutils.tests.common \
    import \
        DistutilsCommandTestBase
from bento.distutils.dist \
    import \
        BentoDistribution
from bento.distutils.commands.build \
    import \
        build
        
class TestBuildCommand(DistutilsCommandTestBase):
    def test_hook(self):
        "Test hook is executed at all in distutils compat mode."
        fid = open(op.join(self.new_cwd, "bento.info"), "wt")
        try:
            fid.write("""\
Name: foo
HookFile: bscript
""")
        finally:
            fid.close()

        fid = open(op.join(self.new_cwd, "bscript"), "wt")
        try:
            fid.write("""\
from bento.commands import hooks

@hooks.pre_build
def pre_build(context):
    raise ValueError()
""")
        finally:
            fid.close()

        dist = BentoDistribution()
        cmd = build(dist)
        cmd.ensure_finalized()
        self.assertRaises(ValueError, cmd.run)



import yaku.utils

def setup(ctx):
    env = ctx.env

    ctx.env["STLINK"] = ["ar"]
    ctx.env["STLINK_TGT_F"] = []
    ctx.env["STLINK_SRC_F"] = []
    ctx.env["STLINKFLAGS"] = ["rcs"]
    ctx.env["STATICLIB_FMT"] = "lib%s.a"

def detect(ctx):
    if yaku.utils.find_program("ar") is None:
        return False
    else:
        return True



import os

from bento.distutils.utils \
    import \
        _is_setuptools_activated
if _is_setuptools_activated():
    from setuptools.command.bdist_egg \
        import \
            bdist_egg as old_bdist_egg
else:
    raise ValueError("You cannot use bdist_egg without setuptools enabled first")

class bdist_egg(old_bdist_egg):
    cmd_name = "build_egg"
    def run(self):
        self.run_command("build")

        cmd_argv = ["--output-dir=%s" % self.dist_dir]
        self.distribution.run_command_in_context(self.cmd_name, cmd_argv)



"""
This module centralizes every internal configuration parameter used throughout
bento.
"""
import os
import sys

import bento

try:
    from bento.__config_py \
        import \
           PKGDATADIR, SITEDIR
except ImportError:
    # Arch-independent path
    PKGDATADIR = os.path.abspath(os.path.dirname(__file__))
    SITEDIR = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))

# Windows binaries
_CLI = os.path.join(PKGDATADIR, "commands", "cli.exe")
WININST_DIR = os.path.join(PKGDATADIR, "commands", "wininst")

# Parser parameters
_PICKLED_PARSETAB = os.path.join(PKGDATADIR, "parsetab")
_OPTIMIZE_LEX = 0
_DEBUG_YACC = 0

# Use subdist bento to avoid clashing with distutils ATM
_SUB_BUILD_DIR = "bento"

CONFIGURED_STATE_DUMP = os.path.join(_SUB_BUILD_DIR, ".config.bin")
DB_FILE = os.path.join(_SUB_BUILD_DIR, "cache.db")
DISTCHECK_DIR = os.path.join(_SUB_BUILD_DIR, "distcheck")
IPKG_PATH = os.path.join(_SUB_BUILD_DIR, "ipkg.info")

BENTO_SCRIPT = "bento.info"

USE_PRIVATE_MODULES = True



import os
import shutil
import tempfile

from bento.compat.api.moves \
    import \
        unittest

from bento._config \
    import \
        IPKG_PATH
from bento.core.node \
    import \
        create_base_nodes
from bento.core.package \
    import \
        PackageDescription
from bento.core \
    import \
        PackageMetadata
from bento.core.pkg_objects \
    import \
        Extension

from bento.commands.egg_utils \
    import \
        EggInfo

DESCR = """\
Name: Sphinx
Version: 0.6.3
Summary: Python documentation generator
Url: http://sphinx.pocoo.org/
DownloadUrl: http://pypi.python.org/pypi/Sphinx
Description: Some long description.
Author: Georg Brandl
AuthorEmail: georg@python.org
Maintainer: Georg Brandl
MaintainerEmail: georg@python.org
License: BSD

Library:
    Packages:
        sphinx,
        sphinx.builders
    Modules:
        cat.py
    Extension: _dog
        Sources: src/dog.c

Executable: sphinx-build
    Module: sphinx
    Function: main
"""

DUMMY_C = r"""\
#include <Python.h>
#include <stdio.h>

static PyObject*
hello(PyObject *self, PyObject *args)
{
    printf("Hello from C\n");
    Py_INCREF(Py_None);
    return Py_None;
}

static PyMethodDef HelloMethods[] = {
    {"hello",  hello, METH_VARARGS, "Print a hello world."},
    {NULL, NULL, 0, NULL}        /* Sentinel */
};

PyMODINIT_FUNC
init%(name)s(void)
{
    (void) Py_InitModule("%(name)s", HelloMethods);
}
"""

def create_fake_package(top_node, packages=None, modules=None, extensions=[]):
    if packages is None:
        packages = []
    if modules is None:
        modules = []
    if extensions is None:
        extensions = []

    for p in packages:
        d = p.replace(".", os.sep)
        n = top_node.make_node(d)
        n.mkdir()
        init = n.make_node("__init__.py")
        init.write("")
    for m in modules:
        d = m.replace(".", os.sep)
        n = top_node.make_node("%s.py" % d)
    for extension in extensions:
        main = extension.sources[0]
        n = top_node.make_node(main)
        n.parent.mkdir()
        n.write(DUMMY_C % {"name": extension.name})
        for s in extension.sources[1:]:
            n = top_node.make_node(s)
            n.write("")

class TestEggInfo(unittest.TestCase):
    def setUp(self):
        self.old_dir = None
        self.tmpdir = None

        self.old_dir = os.getcwd()
        self.tmpdir = tempfile.mkdtemp()

        os.chdir(self.tmpdir)
        self.top_node, self.build_node, self.run_node = \
                create_base_nodes(self.tmpdir, os.path.join(self.tmpdir, "build"))

    def tearDown(self):
        if self.old_dir:
            os.chdir(self.old_dir)
        if os.path.exists(self.tmpdir):
            shutil.rmtree(self.tmpdir)

    def _prepare_egg_info(self):
        create_fake_package(self.top_node, ["sphinx", "sphinx.builders"],
                            ["cat.py"], [Extension("_dog", [os.path.join("src", "dog.c")])])
        ipkg_file = self.build_node.make_node(IPKG_PATH)
        ipkg_file.parent.mkdir()
        ipkg_file.write("")

        files = [os.path.join("sphinx", "builders", "__init__.py"),
                 os.path.join("sphinx", "__init__.py"),
                 os.path.join("src", "dog.c"),
                 os.path.join("cat.py")]

        pkg = PackageDescription.from_string(DESCR)
        meta = PackageMetadata.from_package(pkg)
        executables = pkg.executables

        return EggInfo(meta, executables, files)

    def test_pkg_info(self):
        egg_info = self._prepare_egg_info()
        res = egg_info.get_pkg_info()
        ref = """\
Metadata-Version: 1.1
Name: Sphinx
Version: 0.6.3
Summary: Python documentation generator
Home-page: http://sphinx.pocoo.org/
Author: Georg Brandl
Author-email: georg@python.org
License: BSD
Download-URL: http://pypi.python.org/pypi/Sphinx
Description: Some long description.
Platform: UNKNOWN
"""
        self.assertEqual(res, ref)

    def test_iter_meta(self):
        egg_info = self._prepare_egg_info()
        for name, content in egg_info.iter_meta(self.build_node):
            pass



import sys

from bento.commands.contexts \
    import \
        GlobalContext
from bento.compat.api.moves \
    import \
        unittest
from bento.core.options \
    import \
        PackageOptions
from bento.core.platforms \
    import \
        get_scheme

class TestGlobalContextCustomizedOptions(unittest.TestCase):
    def setUp(self):
        self.context = GlobalContext(None)
        self.maxDiff = None

    def _test(self, package_options, r_scheme):
        self.context.register_package_options(package_options)
        scheme = self.context.retrieve_scheme()

        default_scheme, _ = get_scheme(sys.platform)
        r_scheme.update(default_scheme)
        r_scheme['py_version_short'] = ".".join(str(part) for part in sys.version_info[:2])
        r_scheme['pkgname'] = package_options.name

        self.assertEqual(scheme, r_scheme)

    def test_no_options(self):
        package_options = PackageOptions.from_string("""\
Name: foo
""")
        self._test(package_options, {})

    def test_path_option(self):
        package_options = PackageOptions.from_string("""\
Name: foo

Path: floupi
    Description: yoyo
    Default: /yeah
""")
        self._test(package_options, {"floupi": "/yeah"})



import os
import sys
import tempfile
import shutil

import os.path as op

from bento.compat.api.moves \
    import \
        unittest
from bento.core.package \
    import \
        PackageDescription
from bento.core.pkg_objects \
    import \
        DataFiles
from bento.testing.sub_test_case \
    import \
        SubprocessTestCase
from bento.core.node \
    import \
        create_first_node
from bento.convert.core \
    import \
        monkey_patch, analyse_setup_py, build_pkg, _convert_numpy_data_files, prune_extra_files

class CommonTest(SubprocessTestCase):
    def setUp(self):
        self.save = os.getcwd()
        self.d = tempfile.mkdtemp()
        os.chdir(self.d)
        try:
            self.top_node = create_first_node(self.d)
        except Exception:
            os.chdir(self.save)
            raise

    def tearDown(self):
        os.chdir(self.save)
        shutil.rmtree(self.d)

class TestMonkeyPatch(CommonTest):
    def test_distutils(self):
        monkey_patch(self.top_node, "distutils", "setup.py")
        self.assertTrue("setuptools" not in sys.modules)

    def test_setuptools(self):
        monkey_patch(self.top_node, "setuptools", "setup.py")
        self.assertTrue("setuptools" in sys.modules)

class TestBuildPackage(CommonTest):
    def test_setuptools_include_package(self):
        top = self.top_node

        top.make_node("yeah.txt").write("")
        top.make_node("foo").mkdir()
        top.find_node("foo").make_node("__init__.py").write("")
        top.find_node("foo").make_node("foo.info").write("")
        top.make_node("MANIFEST.in").write("""\
include yeah.txt
include foo/foo.info
""")

        top.make_node("setup.py").write("""\
import setuptools
from distutils.core import setup

setup(name="foo", include_package_data=True, packages=["foo"])
""")

        monkey_patch(top, "setuptools", "setup.py")
        dist, package_objects = analyse_setup_py("setup.py", ["-n", "-q"])
        pkg, options = build_pkg(dist, package_objects, top)

        self.assertEqual(pkg.data_files, {"foo_data": DataFiles("foo_data", ["foo.info"], "$sitedir/foo", "foo")})
        self.assertEqual(pkg.extra_source_files, ["setup.py", "yeah.txt"])

    def test_scripts(self):
        top = self.top_node

        top.make_node("foo").write("")

        top.make_node("setup.py").write("""\
from distutils.core import setup

setup(name="foo", scripts=["foo"])
""")

        monkey_patch(top, "distutils", "setup.py")
        dist, package_objects = analyse_setup_py("setup.py", ["-n", "-q"])
        pkg, options = build_pkg(dist, package_objects, top)

        self.assertEqual(pkg.data_files, {"foo_scripts": DataFiles("foo_scripts", ["foo"], "$bindir", ".")})

class TestMisc(unittest.TestCase):
    def setUp(self):
        super(TestMisc, self).setUp()
        self.save = os.getcwd()
        self.d = tempfile.mkdtemp()
        os.chdir(self.d)
        try:
            self.top_node = create_first_node(self.d)
        except Exception:
            os.chdir(self.save)
            raise

    def tearDown(self):
        os.chdir(self.save)
        shutil.rmtree(self.d)

        super(TestMisc, self).tearDown()

    def test_convert_numpy_data_files(self):
        source_dir = "foo/bar"
        files = [op.join(source_dir, f) for f in ["foufou", "fubar"]]
        for f in files:
            node = self.top_node.make_node(f)
            node.parent.mkdir()
            node.write("")

        pkg_name, source_dir, target_dir, files = _convert_numpy_data_files(self.top_node, source_dir, files)

        self.assertEqual(pkg_name, "foo.bar")

    def test_prune_extra_files(self):
        files = ["doc/foo.info", "yeah.txt", "foo/__init__.py"]
        for f in files:
            n = self.top_node.make_node(f)
            n.parent.mkdir()
            n.write("")

        bento_info = """\
Name: foo

ExtraSourceFiles: yeah.txt

DataFiles: doc
    SourceDir: .
    TargetDir: $sitedir
    Files: doc/foo.info

Library:
    Packages: foo
"""
        pkg = PackageDescription.from_string(bento_info)
        files = prune_extra_files(files, pkg, self.top_node)
        self.assertEqual(files, ["yeah.txt"])



import sys
import os

from bento.core \
    import \
        PackageDescription
from bento.utils.utils \
    import \
        pprint, MODE_755

from bento.core.node \
    import \
        create_root_with_source_tree
from bento.commands.script_utils \
    import \
        create_posix_script, create_win32_script

root = create_root_with_source_tree(os.getcwd(), os.path.join(os.getcwd(), "build"))

def _create_executable(name, executable, scripts_node):
    if sys.platform == "win32":
        nodes = create_win32_script(name, executable, scripts_node)
    else:
        nodes = create_posix_script(name, executable, scripts_node)
        for n in nodes:
            n.chmod(MODE_755)
    return nodes

def install_inplace(pkg):
    """Install scripts of pkg in the current directory."""
    for basename, executable in pkg.executables.items():
        version_str = ".".join([str(i) for i in sys.version_info[:2]])
        scripts_node = root._ctx.srcnode
        for name in [basename, "%s-%s" % (basename, version_str)]:
            nodes = _create_executable(name, executable, scripts_node)
            installed = ",".join([n.path_from(scripts_node) for n in nodes])
            pprint("GREEN", "installing %s in current directory" % installed)

if __name__ == "__main__":
    pkg = PackageDescription.from_file("bento.info")
    if pkg.executables:
        install_inplace(pkg)



import sys

from six.moves import cStringIO

from unittest \
    import \
        TestCase

from bento.parser.nodes \
    import \
        ast_pprint
from bento.parser.parser \
    import \
        parse

class _TestGrammar(TestCase):
    def _test(self, data, expected):
        s = cStringIO()

        p = parse(data)
        ast_pprint(p, string=s)

        try:
            self.assertEqual(s.getvalue(), expected)
        except AssertionError:
            msg = s.getvalue()
            msg += "\n%s" % str(expected)
            raise AssertionError("assertion error:\n%s" % msg)

class TestMeta(_TestGrammar):
    def test_meta_name(self):
        data = "Name: yo"
        expected = """\
Node(type='stmt_list'):
    Node(type='name', value='yo')\
"""

        self._test(data, expected)

    def test_meta_url(self):
        data = "Url: http://example.com"
        expected = """\
Node(type='stmt_list'):
    Node(type='url', value='http://example.com')\
"""

        self._test(data, expected)

    def test_meta_summary(self):
        data = "Summary: a few words of description."
        expected = """\
Node(type='stmt_list'):
    Node(type='summary', value=[Node('literal'), """ \
        """Node('literal'), Node('literal'), Node('literal'), """ \
        """Node('literal'), Node('literal'), Node('literal'), """ \
        """Node('literal'), Node('literal')])"""

        self._test(data, expected)

    def test_meta_author(self):
        data = "Author: John Doe"
        expected = """\
Node(type='stmt_list'):
    Node(type='author', """\
    "value=[Node('literal'), Node('literal'), Node('literal')])"

        self._test(data, expected)

    def test_meta_author_email(self):
        data = "AuthorEmail: john@doe.com"
        expected = """\
Node(type='stmt_list'):
    Node(type='author_email', value='john@doe.com')"""

        self._test(data, expected)

    def test_meta_maintainer_email(self):
        data = "MaintainerEmail: john@doe.com"
        expected = """\
Node(type='stmt_list'):
    Node(type='maintainer_email', value='john@doe.com')"""

        self._test(data, expected)

    def test_meta_maintainer(self):
        data = "Maintainer: John Doe"
        expected = """\
Node(type='stmt_list'):
    Node(type='maintainer', """ \
    "value=[Node('literal'), Node('literal'), Node('literal')])"

        self._test(data, expected)

    def test_meta_license(self):
        data = "License: BSD"
        expected = """\
Node(type='stmt_list'):
    Node(type='license', value='BSD')"""

        self._test(data, expected)

    def test_meta_version(self):
        data = "Version: 1.0"
        expected = """\
Node(type='stmt_list'):
    Node(type='version', value='1.0')\
"""

        self._test(data, expected)

    def test_meta_platforms(self):
        data = "Platforms: any"
        expected = """\
Node(type='stmt_list'):
    Node(type='platforms', value=['any'])"""

        self._test(data, expected)

    def test_meta_classifiers_single_line(self):
        data = "Classifiers: yo"
        expected = """\
Node(type='stmt_list'):
    Node(type='classifiers', value=['yo'])\
"""

        self._test(data, expected)

    def test_meta_classifiers_multi_lines(self):
        data = """\
Classifiers: yo,
    yeah
"""
        expected = """\
Node(type='stmt_list'):
    Node(type='classifiers', value=['yo', 'yeah'])\
"""

        self._test(data, expected)

    def test_meta_classifiers_indent_only(self):
        data = """\
Classifiers:
    yo1,
    yo2\
"""
        expected = """\
Node(type='stmt_list'):
    Node(type='classifiers', value=['yo1', 'yo2'])\
"""

        self._test(data, expected)

    def test_meta_classifiers_full(self):
        data = """\
Classifiers: yo1,
    yo2\
"""
        expected = """\
Node(type='stmt_list'):
    Node(type='classifiers', value=['yo1', 'yo2'])\
"""

        self._test(data, expected)

    def test_meta_stmts(self):
        data = """\
Name: yo
Summary: yeah\
"""
        expected = """\
Node(type='stmt_list'):
    Node(type='name', value='yo')
    Node(type='summary', value=[Node('literal')])\
"""

        self._test(data, expected)

    def test_empty(self):
        data = ""
        expected = "Node(type='empty')"

        self._test(data, expected)

    def test_newline(self):
        data = "\n"
        expected = "Node(type='empty')"

        self._test(data, expected)

    def test_description_single_line(self):
        data = "Description: some words."
        expected = """\
Node(type='stmt_list'):
    Node(type='description', value=[Node('literal'), Node('literal'), Node('literal')])"""

        self._test(data, expected)

    def test_description_simple_indent(self):
        data = """\
Description:
    some words."""
        expected = """\
Node(type='stmt_list'):
    Node(type='description', value=[Node('multi_literal'), Node('multi_literal'), """ \
        """Node('multi_literal')])"""

        self._test(data, expected)

    def test_description_simple_indent_wse(self):
        "Test indented block with ws error."
        data = """\
Description:   
    some words."""
        expected = """\
Node(type='stmt_list'):
    Node(type='description', value=[Node('multi_literal'), Node('multi_literal'), """ \
        """Node('multi_literal')])"""

        self._test(data, expected)

    def test_description_complex_indent(self):
        data = """\
Description:
    some
        indented
            words
    ."""

        expected = """\
Node(type='stmt_list'):
    Node(type='description', value=[Node('multi_literal'), Node('newline'), """ \
        """Node('indent'), """ \
        """Node('multi_literal'), """ \
        """Node('newline'), Node('indent'), """ \
        """Node('multi_literal'), Node('newline'), """ \
        """Node('dedent'), Node('dedent'), """ \
        """Node('multi_literal')])"""

        self._test(data, expected)

class TestConditional(_TestGrammar):
    def test_not_bool(self):
        data = """\
Library:
    if not true:
        Modules: foo.py
"""
        expected = """\
Node(type='stmt_list'):
    Node(type='library'):
        Node(type='library_name', value='default')
        Node(type='library_stmts'):
            Node(type='conditional'):
                Node(type='library_stmts'):
                    Node(type='modules', value=['foo.py'])"""

        self._test(data, expected)

    def test_not_flag(self):
        data = """\
Flag: foo
    Default: true
    Description: yo mama

Library:
    if not flag(foo):
        Modules: foo.py
"""
        expected = """\
Node(type='stmt_list'):
    Node(type='flag'):
        Node(type='flag_declaration', value='foo')
        Node(type='flag_stmts'):
            Node(type='flag_default', value='true')
            Node(type='flag_description', value=[Node('literal'), Node('literal'), Node('literal')])
    Node(type='library'):
        Node(type='library_name', value='default')
        Node(type='library_stmts'):
            Node(type='conditional'):
                Node(type='library_stmts'):
                    Node(type='modules', value=['foo.py'])"""

        self._test(data, expected)

class TestLibrary(_TestGrammar):
    def test_modules(self):
        data = """\
Library:
    Modules: foo.py
"""

        expected = """\
Node(type='stmt_list'):
    Node(type='library'):
        Node(type='library_name', value='default')
        Node(type='library_stmts'):
            Node(type='modules', value=['foo.py'])"""

        self._test(data, expected)

    def test_modules2(self):
        data = """\
Library:
    Modules: foo.py,
        bar.py,
        fubar.py
"""

        expected = """\
Node(type='stmt_list'):
    Node(type='library'):
        Node(type='library_name', value='default')
        Node(type='library_stmts'):
            Node(type='modules', value=['foo.py', 'bar.py', 'fubar.py'])"""

        self._test(data, expected)

    def test_modules3(self):
        data = """\
Library:
    Modules:
        bar.py,
        fubar.py
"""

        expected = """\
Node(type='stmt_list'):
    Node(type='library'):
        Node(type='library_name', value='default')
        Node(type='library_stmts'):
            Node(type='modules', value=['bar.py', 'fubar.py'])"""

        self._test(data, expected)

    def test_packages(self):
        data = """\
Library:
    Packages:
        foo, bar
"""

        expected = """\
Node(type='stmt_list'):
    Node(type='library'):
        Node(type='library_name', value='default')
        Node(type='library_stmts'):
            Node(type='packages', value=['foo', 'bar'])"""

        self._test(data, expected)

    def test_build_requires(self):
        data = """\
Library:
    BuildRequires:
        foo, bar
"""

        expected = """\
Node(type='stmt_list'):
    Node(type='library'):
        Node(type='library_name', value='default')
        Node(type='library_stmts'):
            Node(type='build_requires', value=['foo', 'bar'])"""

        self._test(data, expected)

    def test_build_requires2(self):
        data = """\
Library:
    BuildRequires:
        foo, bar
    BuildRequires: fubar
"""

        expected = """\
Node(type='stmt_list'):
    Node(type='library'):
        Node(type='library_name', value='default')
        Node(type='library_stmts'):
            Node(type='build_requires', value=['foo', 'bar'])
            Node(type='build_requires', value=['fubar'])"""

        self._test(data, expected)

    def test_subdir(self):
        data = """\
Library:
    SubDirectory: lib
"""

        expected = """\
Node(type='stmt_list'):
    Node(type='library'):
        Node(type='library_name', value='default')
        Node(type='library_stmts'):
            Node(type='sub_directory', value='lib')"""

        self._test(data, expected)

class TestExecutable(_TestGrammar):
    def test_simple(self):
        data = """\
Executable: foo
    Module: foo.bar
    Function: main
"""

        expected = """\
Node(type='stmt_list'):
    Node(type='executable'):
        Node(type='exec_name', value='foo')
        Node(type='exec_stmts'):
            Node(type='module', value='foo.bar')
            Node(type='function', value='main')"""

        self._test(data, expected)

class TestPath(_TestGrammar):
    def test_simple(self):
        data = """\
Path: foo
    Default: foo_default
    Description: foo_description
"""

        expected = """\
Node(type='stmt_list'):
    Node(type='path'):
        Node(type='path_declaration', value='foo')
        Node(type='path_stmts'):
            Node(type='path_default', value='foo_default')
            Node(type='path_description', value=[Node('literal')])"""

        self._test(data, expected)

    def test_conditional(self):
        data = """\
Path: foo
    if true:
        Default: foo_default
    Description: foo_description
"""

        expected = """\
Node(type='stmt_list'):
    Node(type='path'):
        Node(type='path_declaration', value='foo')
        Node(type='path_stmts'):
            Node(type='conditional'):
                Node(type='path_stmts'):
                    Node(type='path_default', value='foo_default')
            Node(type='path_description', value=[Node('literal')])"""

        self._test(data, expected)



# setuptools should be first so that bento.distutils knows what to monkey patch
import setuptools
import bento.distutils
bento.distutils.monkey_patch()

if __name__ == '__main__':
    setuptools.setup()



import os
import shutil
import subprocess
import errno

from bento._config \
    import \
        IPKG_PATH
from bento.installed_package_description import \
    InstalledPkgDescription, iter_files

from bento.commands.core import \
    Command, Option
from bento.utils.utils import \
    pprint, extract_exception, MODE_755, MODE_777

def _rollback_operation(line):
    operation, arg = line.split()
    if operation == "MKDIR":
        try:
            os.rmdir(arg)
        except OSError:
            e = extract_exception()
            # FIXME: hardcoded errno !!
            if e.errno != 66:
                raise
    elif operation == "COPY":
        os.remove(arg)
    else:
        raise ValueError("Unknown operation: %s" % operation)

def rollback_transaction(f):
    fid = open(f)
    try:
        lines = fid.readlines()
        for i in range(len(lines)-1, -1, -1):
            _rollback_operation(lines[i])
            lines.pop(i)
    finally:
        fid.close()
        if len(lines) < 1:
            os.remove(f)
        else:
            fid = open(f, "w")
            try:
                fid.writelines(lines)
            finally:
                fid.close()

class TransactionLog(object):
    """Naive version of a journal to rollback interrupted install."""
    def __init__(self, journal_filename):
        if os.path.exists(journal_filename):
            raise IOError("file %s already exists" % journal_filename)
        open(journal_filename, "w").close()
        self.f = open(journal_filename, "w")
        self.journal_filename = journal_filename

    def copy(self, source, target, category):
        if os.path.exists(target):
            self.rollback()
            raise ValueError("File %s already exists, rolled back installation" % target)
        d = os.path.dirname(target)
        if not os.path.exists(d):
            self.makedirs(d)
        self.f.write("COPY %s\n" % target)
        shutil.copy(source, target)
        if category == "executables":
            os.chmod(target, MODE_755)

    def makedirs(self, name, mode=MODE_777):
        head, tail = os.path.split(name)
        if not tail:
            head, tail = os.path.split(head)
        if head and tail and not os.path.exists(head):
            try:
                self.makedirs(head, mode)
            except OSError:
                e = extract_exception()
                if e.errno != errno.EEXIST:
                    raise
            if tail == os.curdir:
                return
        self.mkdir(name, mode)

    def mkdir(self, name, mode=MODE_777):
        self.f.write("MKDIR %s\n" % name) 
        self.f.flush()
        os.mkdir(name, mode)

    def close(self):
        if self.f is not None:
            self.f.close()

    def rollback(self):
        self.f.close()
        self.f = open(self.journal_filename, "r")
        lines = self.f.readlines()
        for line in lines[::-1]:
            _rollback_operation(line.strip())
        self.f = None

def copy_installer(source, target, kind):
    dtarget = os.path.dirname(target)
    if not os.path.exists(dtarget):
        os.makedirs(dtarget)
    shutil.copy(source, target)
    if kind == "executables":
        os.chmod(target, MODE_755)

def unix_installer(source, target, kind):
    if kind in ["executables"]:
        mode = "755"
    else:
        mode = "644"
    cmd = ["install", "-m", mode, source, target]
    strcmd = "INSTALL %s -> %s" % (source, target)
    pprint('GREEN', strcmd)
    if not os.path.exists(os.path.dirname(target)):
        os.makedirs(os.path.dirname(target))
    subprocess.check_call(cmd)

class InstallCommand(Command):
    long_descr = """\
Purpose: install the project
Usage:   bentomaker install [OPTIONS]."""
    short_descr = "install the project."
    common_options = Command.common_options + \
                        [Option("-t", "--transaction",
                                help="Do a transaction-based install", action="store_true"),
                         Option("-n", "--dry-run", "--list-files",
                                help="List installed files (do not install anything)",
                                action="store_true", dest="list_files")]
    def run(self, ctx):
        argv = ctx.command_argv
        p = ctx.options_context.parser
        o, a = p.parse_args(argv)
        if o.help:
            p.print_help()
            return

        n = ctx.build_node.make_node(IPKG_PATH)
        ipkg = InstalledPkgDescription.from_file(n.abspath())
        scheme = ctx.retrieve_configured_scheme()
        ipkg.update_paths(scheme)
        node_sections = ipkg.resolve_paths_with_destdir(ctx.build_node)

        if o.list_files:
            # XXX: this won't take into account action in post install scripts.
            # A better way would be to log install steps and display those, but
            # this will do for now.
            for kind, source, target in iter_files(node_sections):
                print(target.abspath())
            return

        if o.transaction:
            trans = TransactionLog("transaction.log")
            try:
                for kind, source, target in iter_files(node_sections):
                    trans.copy(source.abspath(), target.abspath(), kind)
            finally:
                trans.close()
        else:
            for kind, source, target in iter_files(node_sections):
                copy_installer(source.abspath(), target.abspath(), kind)



import os

import bento.core.node

from bento.errors import \
        InvalidPackage
from bento.core import \
        PackageDescription
from bento.core.pkg_objects import \
        Executable
from bento.utils.utils import \
        is_string

_PKG_TO_DIST = {
        "ext_modules": lambda pkg: [v for v  in \
                                    pkg.extensions.values()],
        "platforms": lambda pkg: [v for v  in pkg.platforms],
        "packages": lambda pkg: [v for v  in pkg.packages],
        "py_modules": lambda pkg: [v for v  in pkg.py_modules],
}

_META_PKG_TO_DIST = {}
def _setup():
    for k in ["name", "url", "author", "author_email", "maintainer",
              "maintainer_email", "license", "download_url"]:
        def _f(attr):
            return lambda pkg: getattr(pkg, attr)
        _META_PKG_TO_DIST[k] = _f(k)
    _META_PKG_TO_DIST["long_description"] = lambda pkg: pkg.description
    _META_PKG_TO_DIST["description"] = lambda pkg: pkg.summary
    def _version_(pkg):
        if pkg.version is None:
            return "UNKNOWN"
        else:
            return pkg.version
    _META_PKG_TO_DIST["version"] = _version_
_setup()
_PKG_TO_DIST.update(_META_PKG_TO_DIST)

def pkg_to_distutils_meta(pkg):
    """Obtain meta data information from pkg into a dictionary which may be
    used directly as an argument for setup function in distutils."""
    d = {}
    for k, v in _META_PKG_TO_DIST.items():
        d[k] = v(pkg)
    return d

def pkg_to_distutils_meta_pkg_info(pkg):
    meta = pkg_to_distutils_meta(pkg)
    meta["summary"] = meta.pop("description")
    meta["description"] = meta.pop("long_description")

    return meta

def pkg_to_distutils(pkg):
    """Convert PackageDescription instance to a dict which may be used
    as argument to distutils/setuptools setup function."""
    d = {}

    for k, v in _PKG_TO_DIST.items():
        d[k] = v(pkg)

    return d

def validate_package(pkg_name, base_node):
    """Given a python package name, check whether it is indeed an existing
    package.

    Package is looked relatively to base_node."""
    # XXX: this function is wrong - use the code from setuptools
    pkg_dir = pkg_name.replace(".", os.path.sep)
    pkg_node = base_node.find_node(pkg_dir)
    if pkg_node is None:
        raise InvalidPackage("directory %s in %s does not exist" % (pkg_dir, base_node.abspath()))
    init = pkg_node.find_node('__init__.py')
    if init is None:
        raise InvalidPackage(
                "Missing __init__.py in package %s (in directory %s)"
                % (pkg_name, base_node.abspath()))
    return pkg_node

def find_package(pkg_name, base_node):
    """Given a python package name, find all its modules relatively to
    base_node."""
    pkg_node = validate_package(pkg_name, base_node)
    ret = []
    for f in pkg_node.listdir():
        if f.endswith(".py"):
            node = pkg_node.find_node(f)
            ret.append(node.path_from(base_node))
    return ret

def validate_packages(pkgs, top):
    ret_pkgs = []
    for pkg in pkgs:
        try:
            validate_package(pkg, top)
        except InvalidPackage:
            # FIXME: add the package as data here
            pass
        else:
            ret_pkgs.append(pkg)
    return ret_pkgs

def distutils_to_package_description(dist):
    root = bento.core.node.Node("", None)
    top = root.find_dir(os.getcwd())

    data = {}

    data['name'] = dist.get_name()
    data['version'] = dist.get_version()
    data['author'] = dist.get_author()
    data['author_email'] = dist.get_author_email()
    data['maintainer'] = dist.get_contact()
    data['maintainer_email'] = dist.get_contact_email()
    data['summary'] = dist.get_description()
    data['description'] = dist.get_long_description().replace("#", "\#")
    data['license'] = dist.get_license()
    data['platforms'] = dist.get_platforms()

    data['download_url'] = dist.get_download_url()
    data['url'] = dist.get_url()

    # XXX: reliable way to detect whether Distribution was monkey-patched by
    # setuptools
    try:
        reqs = getattr(dist, "install_requires")
        # FIXME: how to detect this correctly
        if is_string(reqs):
            reqs = [reqs]
        data['install_requires'] = reqs
    except AttributeError:
        pass

    if dist.py_modules is None:
        data['py_modules'] = []
    else:
        data['py_modules'] = dist.py_modules
    if dist.packages is None:
        packages = []
    else:
        packages = dist.packages
    data['packages'] = validate_packages(packages, top)
    if dist.ext_modules:
        data['extensions'] = dict([(e.name, e) for e in dist.ext_modules])
    else:
        data['extensions'] = {}
    data['classifiers'] = dist.get_classifiers()

    data["executables"] = {}

    entry_points = entry_points_from_dist(dist)
    if entry_points:
        console_scripts = entry_points.get("console_scripts", [])
        for entry in console_scripts:
            exe = Executable.from_representation(entry)
            data["executables"][exe.name] = exe

    return PackageDescription(**data)

_DIST_CONV_DICT = {
    "long_description": lambda meta: meta.description,
    "description": lambda meta: meta.summary,
    # TODO: keywords not implemented yet
    "keywords": lambda meta: [],
    "fullname": lambda meta: "%s-%s" % (meta.name, meta.version),
    "contact": lambda meta: (meta.maintainer or
                             meta.author or
                             "UNKNOWN"),
    "contact_email": lambda meta: (meta.maintainer_email or
                                   meta.author_email or
                                   "UNKNOWN"),
    "requires": lambda meta: meta.install_requires,
    "provides": lambda meta: [],
    "obsoletes": lambda meta: []
}

def to_distutils_meta(meta):
    from bento.compat.dist \
        import \
            DistributionMetadata
    ret = DistributionMetadata()
    for m in ret._METHOD_BASENAMES:
        try:
            val = _DIST_CONV_DICT[m](meta)
        except KeyError:
            val = getattr(meta, m)
        setattr(ret, m, val)

    return ret

def write_pkg_info(pkg, file):
    dist_meta = to_distutils_meta(pkg)
    dist_meta.write_pkg_file(file)

def entry_points_from_dist(dist):
    if hasattr(dist, "entry_points"):
        from pkg_resources import split_sections
        if is_string(dist.entry_points):
            entry_points = {}
            sections = split_sections(dist.entry_points)
            for group, lines in sections:
                group = group.strip()
                entry_points[group] = lines
        else:
            entry_points = dist.entry_points
    else:
        entry_points = {}
    return entry_points



"""Global configuration

__file__, etc... are only allowed in this module.
"""
import sys

from os.path \
    import \
        join, abspath, dirname

TOOLDIRS = [abspath(join(dirname(__file__), "tools"))]

# relative to the build directory
DEFAULT_ENV = "default.env.py"
BUILD_CONFIG = "build.config.py"
HOOK_DUMP = ".hooks.pck"

CONFIG_CACHE = ".config.pck"
BUILD_CACHE = ".build.pck"

_OUTPUT = sys.stdout



import os
import sys
import distutils.unixccompiler
import distutils.ccompiler
import distutils.sysconfig
import distutils.command.build_ext as build_ext
import distutils.dist as dist

from distutils \
    import \
        errors
from distutils \
    import \
        sysconfig

DEFAULT_COMPILERS = {
        "win32": [None, "mingw32"],
        "default": [None]
}

def _mingw32_cc():
    compiler_type = "mingw32"
    compiler = distutils.ccompiler.new_compiler(compiler=compiler_type)
    return compiler.compiler_so

def detect_distutils_cc(ctx):
    if not sys.platform in DEFAULT_COMPILERS:
        plat = "default"
    else:
        plat = sys.platform

    sys.stderr.write("Detecting distutils C compiler... ")
    compiler_type = \
            distutils.ccompiler.get_default_compiler()
    if sys.platform == "win32":
        if compiler_type == "msvc":
            try:
                compiler = distutils.ccompiler.new_compiler(
                        compiler="msvc")
                compiler.initialize()
                cc = [compiler.cc]
            except errors.DistutilsPlatformError:
                cc = _mingw32_cc()
        else:
            cc = _mingw32_cc()
    else:
        cc = distutils.sysconfig.get_config_var("CC")
        # FIXME: use shlex for proper escaping handling
        cc = cc.split()
    sys.stderr.write("%s\n" % compiler_type)
    return cc

# XXX: unixccompiler instances are the only classes where we can hope
# to get semi-sensical data. Reusing them also makes transition easier
# for packagers, as their compilation options will be reused.
# OTOH, having a sane tooling system will make customization much
# easier.
def get_configuration(compiler_type=None):
    plat = os.name
    if compiler_type is None:
        compiler_type = distutils.ccompiler.get_default_compiler(plat)

    if not compiler_type in distutils.ccompiler.compiler_class:
        raise ValueError("compiler type %s is not recognized" %
                         compiler_type)

    env = {"CC": [],
        "CPPPATH": [],
        "BASE_CFLAGS": [],
        "OPT": [],
        "SHARED": [],
        "CFLAGS": [],
        "SHLINK": [],
        "LDFLAGS": [],
        "LIBDIR": [],
        "LIBS": [],
        "SO": ""}

    env["CPPPATH"].append(sysconfig.get_python_inc())
    if compiler_type == "unix":
        env["CC"].extend(sysconfig.get_config_var("CC").split(" "))
        env["BASE_CFLAGS"].extend(sysconfig.get_config_var("BASECFLAGS").split(" "))
        env["OPT"].extend(sysconfig.get_config_var("OPT").split(" "))
        env["SHARED"].extend(sysconfig.get_config_var("CCSHARED").split(" "))

        env["SHLINK"] = sysconfig.get_config_var("LDSHARED").split(" ")
        env["SO"] = sysconfig.get_config_var("SO")
        env["LDFLAGS"] = sysconfig.get_config_var("LDFLAGS").split()
        if "-pthread" in sysconfig.get_config_var("LDFLAGS"):
            env["LDFLAGS"].insert(0, "-pthread")
        env["CFLAGS"].extend(sysconfig.get_config_var("CFLAGS").split(" "))
        env["FRAMEWORKS"] = []
        setup_unix(env)
    elif compiler_type == "msvc":
        setup_msvc(env)
    elif compiler_type == "mingw32":
        setup_mingw32(env)
    else:
        raise ValueError("Gne ?")

    return env

def _get_ext_library_dirs():
    binst = build_ext.build_ext(dist.Distribution())
    binst.initialize_options()
    binst.finalize_options()
    return binst.library_dirs

def _get_ext_libraries(compiler):
    binst = build_ext.build_ext(dist.Distribution())
    binst.compiler = compiler
    binst.initialize_options()
    binst.finalize_options()
    class _FakeExt(object):
        def __init__(self):
            self.libraries = []
    return binst.get_libraries(_FakeExt())

def setup_unix(env):
    if sys.platform == "darwin":
        env["LDFLAGS"].extend(["-bundle", "-undefined", "dynamic_lookup"])

        def _strip_arch(flag):
            value = env[flag]
            while "-arch" in value:
                id = value.index("-arch")
                value.pop(id)
                value.pop(id)
            return value
        for flag in ["BASE_CFLAGS", "LDFLAGS"]:
            env[flag] = _strip_arch(flag)

def setup_msvc(env):
    compiler = distutils.ccompiler.new_compiler(
            compiler="msvc")
    compiler.initialize()

    env["CC"] = compiler.cc
    env["BASE_CFLAGS"].extend(compiler.compile_options)

    env["SHLINK"] = compiler.linker
    env["SO"] = ".pyd"
    env["LDFLAGS"] = compiler.ldflags_shared
    env["LIBDIR"].extend( _get_ext_library_dirs())

def setup_mingw32(env):
    compiler = distutils.ccompiler.new_compiler(
            compiler="mingw32")

    env["CC"] = ["gcc"]
    env["BASE_CFLAGS"].extend(["-mno-cygwin"])

    env["SHLINK"] = ["gcc", "-mno-cygwin", "-shared"]
    env["SO"] = ".pyd"
    #env["LDFLAGS"] = compiler.ldflags_shared
    env["LIBDIR"].extend( _get_ext_library_dirs())

    libs = _get_ext_libraries(compiler)
    libs += compiler.dll_libraries 
    env["LIBS"].extend(libs)



import tarfile

import os.path as op

import bento.compat.api as compat
import bento.errors

from bento.commands.core \
    import \
        Command, Option

def archive_basename(pkg):
    if pkg.version:
        return "%s-%s" % (pkg.name, pkg.version)
    else:
        return pkg.name

def create_tarball(node_pkg, archive_root, archive_node):
    tf = tarfile.open(archive_node.abspath(), "w:gz")
    try:
        for filename, alias in node_pkg.iter_source_files():
            tf.add(filename, op.join(archive_root, alias))
    finally:
        tf.close()

def create_zarchive(node_pkg, archive_root, archive_node):
    zid = compat.ZipFile(archive_node.abspath(), "w", compat.ZIP_DEFLATED)
    try:
        for filename, alias in node_pkg.iter_source_files():
            zid.write(filename, op.join(archive_root, alias))
    finally:
        zid.close()

_FORMATS = {"gztar": {"ext": ".tar.gz", "func": create_tarball},
            "zip": {"ext": ".zip", "func": create_zarchive}}

def create_archive(archive_name, archive_root, node_pkg, top_node, run_node, format="tgz", output_directory="dist"):
    if not format in _FORMATS:
        raise ValueError("Unknown format: %r" % (format,))

    archive_node = top_node.make_node(op.join(output_directory, archive_name))
    archive_node.parent.mkdir()

    _FORMATS[format]["func"](node_pkg, archive_root, archive_node) 
    return archive_root, archive_node

class SdistCommand(Command):
    long_descr = """\
Purpose: create a tarball for the project
Usage:   bentomaker sdist [OPTIONS]."""
    short_descr = "create a tarball."
    common_options = Command.common_options \
                        + [Option("--output-dir",
                                  help="Output directory", default="dist"),
                           Option("--format",
                                  help="Archive format (supported: 'gztar', 'zip')", default="gztar"),
                           Option("--output-file",
                                  help="Archive filename (default: $pkgname-$version.$archive_extension)")]

    def run(self, ctx):
        argv = ctx.command_argv
        p = ctx.options_context.parser
        o, a =  p.parse_args(argv)
        if o.help:
            p.print_help()
            return

        pkg = ctx.pkg
        format = o.format

        archive_root = "%s-%s" % (pkg.name, pkg.version)
        if not o.output_file:
            archive_name = archive_basename(pkg) + _FORMATS[format]["ext"]
        else:
            output = op.basename(o.output_file)
            if output != o.output_file:
                raise bento.errors.BentoError("Invalid output file: should not contain any directory")
            archive_name = output

        # XXX: find a better way to pass archive name from other commands (used
        # by distcheck ATM)
        self.archive_root, self.archive_node = create_archive(archive_name, archive_root, ctx._node_pkg,
                ctx.top_node, ctx.run_node, o.format, o.output_dir)



from waflib import Task
from waflib.TaskGen import before_method, feature, taskgen_method
from waflib.Tools import ccroot

import waflib.Tools.c

def colon(env, var1, var2):
	tmp = env[var1]
	if isinstance(var2, str):
		it = env[var2]
	else:
		it = var2
	if isinstance(tmp, str):
		return [tmp % x for x in it]
	else:
		lst = []
		for y in it:
			lst.extend(tmp)
			lst.append(y)
		return lst

@taskgen_method
def interpolate_flags(task_gen, line):
	env = task_gen.env
	reg_act = Task.reg_act

	extr = []
	def repl(match):
		g = match.group
		if g('dollar'): return "$"
		elif g('backslash'): return '\\\\'
		elif g('subst'): extr.append((g('var'), g('code'))); return "%s"
		return None

	line = reg_act.sub(repl, line) or line

	ret = []
	for (var, meth) in extr:
		if meth:
			if meth.startswith(':'):
				m = meth[1:]
				ret.extend(colon(env, var, m))
			else:
				raise NotImplementedError()
				ret.append("%s%s" % (var, meth))
		else:
			ret.extend(env[var])
	return ret

class c(waflib.Tools.c.c):
	run_str = '${CC} ${ARCH_ST:ARCH} ${CFLAGS} ${CPPFLAGS} ${FRAMEWORKPATH_ST:FRAMEWORKPATH} ${CPPPATH_ST:INCPATHS} ${DEFINES_ST:DEFINES} ${CC_SRC_F}${SRC} ${CC_TGT_F}${TGT} ${ORDERED_CFLAGS}'

class cprogram(waflib.Tools.c.cprogram):
	run_str = '${LINK_CC} ${LINKFLAGS} ${CCLNK_SRC_F}${SRC} ${CCLNK_TGT_F}${TGT[0].abspath()} ${RPATH_ST:RPATH} ${FRAMEWORKPATH_ST:FRAMEWORKPATH} ${FRAMEWORK_ST:FRAMEWORK} ${ARCH_ST:ARCH} ${STLIB_MARKER} ${STLIBPATH_ST:STLIBPATH} ${STLIB_ST:STLIB} ${SHLIB_MARKER} ${LIBPATH_ST:LIBPATH} ${LIB_ST:LIB} ${ORDERED_LINKFLAGS}'

class cshlib(cprogram, waflib.Tools.c.cprogram):
    pass

@feature("cprogram", "cshlib", "cstlib", "python")
@before_method("propagate_uselib_vars")
def apply_ordered_flags(self):
	link_flag_name_to_template = {
		"FRAMEWORK": "${FRAMEWORK_ST:%s}",
		"LIB": "${LIB_ST:%s}",
		"LIBPATH": "${LIBPATH_ST:%s}",
		"LINKFLAGS": "${%s}",
		"STLIB": "${STLIB_ST:%s}"
    }

	c_flag_name_to_template = {
		"ARCH": "${ARCH_ST:%s}",
		"CFLAGS": "${%s}",
		"CPPFLAGS": "${%s}",
		"FRAMEWORKPATH": "${FRAMEWORKPATH_ST:%s}",
		"CPPPATH": "${CPPPATH_ST:%s}",
		"DEFINES": "${DEFINES_ST:%s}",
		"INCLUDES": "${CPPPATH_ST:%s}",
	}

	# FIXME: use correct uselib_vars
	uselib_vars = set()
	if "cprogram" in self.features:
		uselib_vars |= ccroot.USELIB_VARS["cprogram"]
	if "cshlib" in self.features:
		uselib_vars |= ccroot.USELIB_VARS["cshlib"]
	#if "c" in self.features:
	#	uselib_vars |= ccroot.USELIB_VARS["c"]
	#if "pyext" in self.features:
	#	uselib_vars |= ccroot.USELIB_VARS["pyext"]
	env = self.env

	uselibs = self.to_list(getattr(self, 'uselib', []))
	ordered_link = []
	ordered_c = []

	to_remove = []

	for uselib in uselibs:
		for uselib_var in uselib_vars:
			key = uselib_var + '_' + uselib
			value = env[key]
			if value:
				to_remove.append(key)
				if uselib_var in link_flag_name_to_template:
					flag_template = link_flag_name_to_template[uselib_var]
					ordered_link.append(flag_template % key)
				elif uselib_var in c_flag_name_to_template:
					flag_template = c_flag_name_to_template[uselib_var]
					ordered_c.append(flag_template % key)
				else:
					raise NotImplementedError(uselib_var)
	ordered_linkflags = self.interpolate_flags(" ".join(ordered_link))
	env["ORDERED_LINKFLAGS"] = ordered_linkflags
	#ordered_cflags = self.interpolate_flags(" ".join(ordered_c))
	#env["ORDERED_CFLAGS"] = ordered_cflags

	for key in to_remove:
		del env[key]



import os
import tempfile

from bento._config \
    import \
        IPKG_PATH
from bento.commands.core \
    import \
        Command, Option
from bento.commands.egg_utils \
    import \
        EggInfo, egg_info_dirname
from bento.commands.wininst_utils \
    import \
        wininst_filename, create_exe
from bento.core \
    import \
        PackageMetadata
from bento.installed_package_description \
    import \
        InstalledPkgDescription, iter_files

import bento.compat.api as compat
import bento.utils.path

class BuildWininstCommand(Command):
    long_descr = """\
Purpose: build wininst
Usage:   bentomaker build_wininst [OPTIONS]"""
    short_descr = "build wininst."
    common_options = Command.common_options \
                        + [Option("--output-dir",
                                  help="Output directory", default="dist"),
                           Option("--output-file",
                                  help="Output filename")]

    def run(self, ctx):
        argv = ctx.command_argv
        p = ctx.options_context.parser
        o, a = p.parse_args(argv)
        if o.help:
            p.print_help()
            return

        n = ctx.build_node.make_node(IPKG_PATH)
        ipkg = InstalledPkgDescription.from_file(n.abspath())
        create_wininst(ipkg, src_root_node=ctx.build_node, build_node=ctx.build_node,
                       wininst=o.output_file,
                       output_dir=o.output_dir)

def create_wininst(ipkg, src_root_node, build_node, egg_info=None, wininst=None, output_dir=None):
    meta = PackageMetadata.from_ipkg(ipkg)
    if egg_info is None:
        egg_info = EggInfo.from_ipkg(ipkg, build_node)

    # XXX: do this correctly, maybe use same as distutils ?
    if wininst is None:
        wininst = os.path.join(output_dir, wininst_filename(meta))
    else:
        wininst = os.path.join(output_dir, wininst)
    bento.utils.path.ensure_dir(wininst)

    egg_info_dir = os.path.join("PURELIB", egg_info_dirname(meta.fullname))

    fid, arcname = tempfile.mkstemp(prefix="zip")
    zid = compat.ZipFile(arcname, "w", compat.ZIP_DEFLATED)
    try:
        for filename, cnt in egg_info.iter_meta(build_node):
            zid.writestr(os.path.join(egg_info_dir, filename), cnt)

        wininst_paths = compat.defaultdict(lambda: r"DATA\share\$pkgname")
        wininst_paths.update({"bindir": "SCRIPTS", "sitedir": "PURELIB",
                              "gendatadir": "$sitedir"})
        d = {}
        for k in ipkg._path_variables:
            d[k] = wininst_paths[k]
        ipkg.update_paths(d)
        file_sections = ipkg.resolve_paths(src_root_node)

        def write_content(source, target, kind):
            zid.write(source.abspath(), target.abspath())

        for kind, source, target in iter_files(file_sections):
            write_content(source, target, kind)

    finally:
        zid.close()
        os.close(fid)

    create_exe(ipkg, arcname, wininst)



import os
import sys
try:
    from hashlib import md5
except ImportError:
    from md5 import md5
import subprocess

if sys.version_info[0] < 3:
    from cPickle \
        import \
            dumps
else:
    from pickle \
        import \
            dumps

from yaku.pprint \
    import \
        pprint
from yaku.utils \
    import \
        get_exception, is_string, function_code
from yaku.errors \
    import \
        TaskRunFailure, WindowsError

# TODO:
#   - factory for tasks, so that tasks can be created from strings
#   instead of import (import not extensible)

# Task factory - taken from waf, to avoid metaclasses magic
class _TaskFakeMetaclass(type):
    def __init__(cls, name, bases, d):
        super(_TaskFakeMetaclass, cls).__init__(name, bases, d)
        name = cls.__name__

_CLASSES = {}
def task_factory(name):
    global _CLASSES
    try:
        klass = _CLASSES[name]
    except KeyError:
        klass = _TaskFakeMetaclass('%sTask' % name, (_Task,), {})
        klass.name = name
        _CLASSES[name] = klass
    return klass

base = _TaskFakeMetaclass('__task_base', (object,), {})

class _Task(object):
    before = []
    after = []
    def __init__(self, outputs, inputs, func=None, deps=None, env=None, env_vars=None):
        if is_string(inputs):
            self.inputs = [inputs]
        else:
            self.inputs = inputs
        if is_string(outputs):
            self.outputs = [outputs]
        else:
            self.outputs = outputs
        self.uid = None
        self.func = func
        if deps is None:
            self.deps = []
        else:
            self.deps = deps
        self.cache = None
        self.env = env
        self.env_vars = env_vars
        self.scan = None
        self.disable_output = False
        self.log = None

    # UID and signature functionalities
    #----------------------------------
    def get_uid(self):
        if self.uid is None:
            m = md5()
            up = m.update
            up(self.__class__.__name__.encode())
            for x in self.inputs + self.outputs:
                up(x.abspath().encode())
            self.uid = m.digest()
        return self.uid

    def signature(self):
        if self.cache is None:
            sig = self._signature()
            self.cache = sig
            return sig
        else:
            return self.cache

    def _signature(self):
        m = md5()

        self._sig_explicit_deps(m)
        for k in self.env_vars:
            m.update(dumps(self.env[k]))
        if self.func:
            m.update(function_code(self.func).co_code)
        return m.digest()

    def _sig_explicit_deps(self, m):
        for s in self.inputs + self.deps:
            #if os.path.exists(s):
            #    m.update(open(s).read())
            m.update(s.read(flags="rb"))
        return m.digest()
        
    # execution
    #----------
    def run(self):
        self.func(self)

    def exec_command(self, cmd, cwd, env=None):
        if cwd is None:
            cwd = self.gen.bld.bld_root.abspath()
        kw = {}
        if env is not None:
            kw["env"] = env
        if not self.disable_output:
            if self.env["VERBOSE"]:
                pprint('GREEN', " ".join([str(c) for c in cmd]))
            else:
                pprint('GREEN', "%-16s%s" % (self.name.upper(), " ".join([i.bldpath() for i in self.inputs])))

        self.gen.bld.set_cmd_cache(self, cmd)
        try:
            p = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                    stderr=subprocess.STDOUT, cwd=cwd, **kw)
            stdout = p.communicate()[0].decode("utf-8")
            if p.returncode:
                raise TaskRunFailure(cmd, stdout)
            if sys.version_info >= (3,):
                stdout = stdout
            else:
                stdout = stdout.encode("utf-8")
            if self.disable_output:
                self.log.write(stdout)
            else:
                sys.stderr.write(stdout)
            self.gen.bld.set_stdout_cache(self, stdout)
        except OSError:
            e = get_exception()
            raise TaskRunFailure(cmd, str(e))
        except WindowsError:
            e = get_exception()
            raise TaskRunFailure(cmd, str(e))

    def __repr__(self):
        ins = ",".join([i.name for i in self.inputs])
        outs = ",".join([i.name for i in self.outputs])
        return "'%s: %s -> %s'" % (self.name, ins, outs)



import os
import sys
import re
import shlex

GCC_DRIVER_LINE = re.compile('^Driving:')
POSIX_STATIC_EXT = re.compile('\S+\.a')
POSIX_LIB_FLAGS = re.compile('-l\S+')

def is_output_verbose(out):
    for line in out.splitlines():
        if not GCC_DRIVER_LINE.search(line):
            if POSIX_STATIC_EXT.search(line) or POSIX_LIB_FLAGS.search(line):
                return True
    return False

# linkflags which match those are ignored
LINKFLAGS_IGNORED = [r'-lang*', r'-lcrt[a-zA-Z0-9]*\.o', r'-lc$', r'-lSystem',
                     r'-libmil', r'-LIST:*', r'-LNO:*']
if os.name == 'nt':
    LINKFLAGS_IGNORED.extend([r'-lfrt*', r'-luser32',
            r'-lkernel32', r'-ladvapi32', r'-lmsvcrt',
            r'-lshell32', r'-lmingw', r'-lmoldname'])
else:
    LINKFLAGS_IGNORED.append(r'-lgcc*')

RLINKFLAGS_IGNORED = [re.compile(f) for f in LINKFLAGS_IGNORED]

def _match_ignore(line):
    """True if the line should be ignored."""
    if [i for i in RLINKFLAGS_IGNORED if i.match(line)]:
        return True
    else:
        return False

def parse_flink(output):
    """Given the output of verbose link of fortran compiler, this
    returns a list of flags necessary for linking using the standard
    linker."""
    # TODO: On windows ?
    final_flags = []
    for line in output.splitlines():
        if not GCC_DRIVER_LINE.match(line):
            _parse_f77link_line(line, final_flags)
    return final_flags

SPACE_OPTS = re.compile('^-[LRuYz]$')
NOSPACE_OPTS = re.compile('^-[RL]')

def _parse_f77link_line(line, final_flags):
    #line = line.encode("utf-8")
    lexer = shlex.shlex(line, posix = True)
    lexer.whitespace_split = True

    t = lexer.get_token()
    tmp_flags = []
    while t:
        def parse(token):
            # Here we go (convention for wildcard is shell, not regex !)
            #   1 TODO: we first get some root .a libraries
            #   2 TODO: take everything starting by -bI:*
            #   3 Ignore the following flags: -lang* | -lcrt*.o | -lc |
            #   -lgcc* | -lSystem | -libmil | -LANG:=* | -LIST:* | -LNO:*)
            #   4 take into account -lkernel32
            #   5 For options of the kind -[[LRuYz]], as they take one argument
            #   after, the actual option is the next token 
            #   6 For -YP,*: take and replace by -Larg where arg is the old
            #   argument
            #   7 For -[lLR]*: take

            # step 3
            if _match_ignore(token):
                pass
            # step 4
            elif token.startswith('-lkernel32') and sys.platform == 'cygwin':
                tmp_flags.append(token)
            # step 5
            elif SPACE_OPTS.match(token):
                t = lexer.get_token()
                # FIXME: this does not make any sense ... pull out
                # what we need from this section
                #if t.startswith('P,'):
                #    t = t[2:]
                #    for opt in t.split(os.pathsep):
                #        tmp_flags.append('-L%s' % opt)
            # step 6
            elif NOSPACE_OPTS.match(token):
                tmp_flags.append(token)
            # step 7
            elif POSIX_LIB_FLAGS.match(token):
                tmp_flags.append(token)
            else:
                # ignore anything not explicitely taken into account
                pass

            t = lexer.get_token()
            return t
        t = parse(t)

    final_flags.extend(tmp_flags)
    return final_flags



import operator
import tarfile

class TarFile(tarfile.TarFile):
    # Add extractall method to tarfile.TarFile (python 2.4*). Copied
    # from upstream v2.6.5
    def extractall(self, path=".", members=None):
        """Extract all members from the archive to the current working
           directory and set owner, modification time and permissions on
           directories afterwards. `path' specifies a different directory
           to extract to. `members' is optional and must be a subset of the
           list returned by getmembers().
        """
        directories = []

        if members is None:
            members = self

        for tarinfo in members:
            if tarinfo.isdir():
                # Extract directories with a safe mode.
                directories.append(tarinfo)
                tarinfo = copy.copy(tarinfo)
                tarinfo.mode = 0700
            self.extract(tarinfo, path)

        # Reverse sort directories.
        directories.sort(key=operator.attrgetter('name'))
        directories.reverse()

        # Set correct owner, mtime and filemode on directories.
        for tarinfo in directories:
            dirpath = os.path.join(path, tarinfo.name)
            try:
                self.chown(tarinfo, dirpath)
                self.utime(tarinfo, dirpath)
                self.chmod(tarinfo, dirpath)
            except ExtractError, e:
                if self.errorlevel > 1:
                    raise
                else:
                    self._dbg(1, "tarfile: %s" % e)
